/************************************************************************
 *
 * Constants and vars
 *
 ************************************************************************/
const IMGLIKEOPERA_CONTRACTID   = "@mozilla.org/imglikeopera;1";
const IMGLIKEOPERA_CID          = Components.ID("{9aa46f4f-4dc7-4c06-97af-5035170633fe}");

const CATMAN_CONTRACTID         = "@mozilla.org/categorymanager;1";
const PREFSERVICE_CONTRACTID    = "@mozilla.org/preferences-service;1";

const PREFSERVICE_BRANCH        = Components.classes[PREFSERVICE_CONTRACTID]
                                            .getService(Components.interfaces.nsIPrefService)
                                            .getBranch("imglikeopera.");

//~ nsIContentPolicy constants
const REJECT_REQUEST            = Components.interfaces.nsIContentPolicy.REJECT_REQUEST;
const REJECT_TYPE               = Components.interfaces.nsIContentPolicy.REJECT_TYPE;
const REJECT_SERVER             = Components.interfaces.nsIContentPolicy.REJECT_SERVER;
const REJECT_OTHER              = Components.interfaces.nsIContentPolicy.REJECT_OTHER;
const ACCEPT                    = Components.interfaces.nsIContentPolicy.ACCEPT;

const TYPE_IMAGE                = Components.interfaces.nsIContentPolicy.TYPE_IMAGE;
const TYPE_OBJECT               = Components.interfaces.nsIContentPolicy.TYPE_OBJECT;

//~ cache stuff
const CACHESERVICE              = Components.classes["@mozilla.org/network/cache-service;1"]
                                            .getService(Components.interfaces.nsICacheService);

var httpCacheSession            = CACHESERVICE.createSession("HTTP", Components.interfaces
                                              .nsICache.STORE_ANYWHERE, true);

httpCacheSession.doomEntriesIfExpired = false;

/**
    Note: During Gecko 1.8 development, nsIPrefBranchInternal was renamed to
    nsIPrefBranch2 [https://bugzilla.mozilla.org/show_bug.cgi?id=281414] and
    was frozen. nsIPrefBranchInternal name is still supported in Gecko 1.8,
    so this is what you should use in extensions that need to be compatible
    with Gecko 1.7 and Gecko 1.8 (Firefox 1.0/1.1).
    For newer extensions use nsIPrefBranch2.
**/

//~ for ff1.0 version; bad way..., bad fox...
//~ http://imfo.ru/archive/2004/09/22/firefox_back_to_the_future
//~ update: argh! trunk 1.1 buggy too!
const WINDOWMEDIATOR = Components.classes["@mozilla.org/appshell/window-mediator;1"].
        getService(Components.interfaces.nsIWindowMediator);

/************************************************************************
 *
 * Class definition
 *
 ************************************************************************/

var nsImgLikeOpera =
{
    shouldLoad: function(contentType, contentLocation, context, obj)
    {
        if ((contentType == TYPE_IMAGE) &&
            (contentLocation.scheme == "http" || contentLocation.scheme == "https"))
        {
            var docPolicy = null; //~ not realy(?) need, remove later
            
            //~ I can write this extension for FF!=1.0, but not now.
            //~ ... so, catch images from styles
            try {
                var doc = obj.ownerDocument.defaultView.top.document;
            }
            catch(ex)
            {
                //~ check requested URL in USER filters
                var catchedByRule = checkUrlBlockByFilter(contentLocation.spec);
                
                //~ need default policy...
                docPolicy = catchedByRule ? catchedByRule : 3;
                
                //~ skip cache searching
                if (docPolicy == 4) return REJECT_OTHER;
                
                //~ Searching in cache.
                var expire = urlInCache(contentLocation.spec, docPolicy);
                
                if (catchedByRule) {
                    switch (docPolicy) {
                        case 1:
                        case 2:
                            //~ "case: 2" also return true (unknown document location, FF bug)
                            return ACCEPT;
                            break;
                        case 3:
                            return expire ? REJECT_OTHER : ACCEPT;
                            break;
                    }
                }
                
                //~ No filters for this url.
                return expire ? iloSettingsLoadFromCss : ACCEPT;
            }
            
            /*** A LOT OF BIG WORK/reWRITE HERE! ***/
            /*** ******************************* ***/
            //if (!obj.src) return ACCEPT;
            if (!obj.getAttribute("src") &&
                obj.tagName.toLowerCase() != "img")
                return ACCEPT;
            
            
            //~ _SECOND_ REQUEST
            //~ 1) Double request, ugly, but need for placeholders. (2do: better way[?])
            //~ 2) DHTML stuff blocked here (mouseOvers on blocked images with they "src" change; etc.)
            if (obj.iloFixP && obj.getAttribute("ilo-ph-fix"))
            {
                if (obj.iloFixP == "fixed")
                    return blockObj(obj, contentLocation);
                
                //~ iloFixP == "2fix"
                obj.iloFixP = "fixed"; //~ return true;
                
                obj.currentURI = contentLocation;
                
                /*** ...more usability, more and more and more... ***/
                
                return ACCEPT;
            }
            
            obj.setAttribute("ilo-full-src", contentLocation.spec);
            
            //~ search or set document policy
            if (!doc.iloDocPolicy) {
                var enumerator = WINDOWMEDIATOR.getEnumerator(null);
                while (enumerator.hasMoreElements()) {
                    try { //~ try/catch need for JSConsole, etc.
                        var tabs = enumerator.getNext().window.getBrowser().browsers;
                        var tabsCount = tabs.length;
                        for (var i=0; i<tabsCount; i++) {
                            if (doc == tabs[i].contentDocument) {
                                if (!tabs[i].iloTabPolicy) {
                                    //~ 2DO: remove this "if", 'coz new tabs has
                                    //~ policy from imglikeopera.js?
                                    var iloSettingsDefaultPolicy = 3;
                                    tabs[i].iloTabPolicy = iloSettingsDefaultPolicy;
                                    doc.iloDocPolicy = iloSettingsDefaultPolicy;
                                }
                                else {
                                    doc.iloDocPolicy = tabs[i].iloTabPolicy;
                                    doc.iloUrlsBlock = "";
                                    doc.iloUrlsPass = "";
                                }
                            }
                        }
                    } catch(e){}
                }
            }
            
            if (!doc.iloDocPolicy) return ACCEPT; //~ smthng wrng
            docPolicy = doc.iloDocPolicy;
            
            var re = new RegExp(contentLocation.spec.replace(/(\.|\||\+|\?|\$|\(|\)|\[|\])/g,"\\\$1")+" ");
            
            if (re.test( doc.iloUrlsBlock )) return blockObj(obj, contentLocation);
            else if (re.test( doc.iloUrlsPass )) return passObj(obj);
            
            //~ check requested URL in USER filters
            var catchedByRule = checkUrlBlockByFilter(contentLocation.spec);
            if (catchedByRule) docPolicy = catchedByRule;
            
            //~ "Don't load images" => skip work with cache, return false.
            if (docPolicy == 4) return blockObj(obj, contentLocation);
            
            var expire = urlInCache(contentLocation.spec, docPolicy);
            var passIt = true;
            
            switch (docPolicy) {
                case 1:
                    break;
                case 2:
                    re = new RegExp(doc.location.host.replace(/^www\./i,""), "i");
                    passIt = re.test(contentLocation.host) ? true : false;
                    break;
                case 3:
                    passIt = expire ? false : true;
                    break;
                //~ case 4:
                    //~ passIt = false;
                    //~ break;
            }
            
            switch (passIt) {
                case false:
                    doc.iloUrlsBlock += contentLocation.spec + " ";
                    return blockObj(obj, contentLocation);
                    break;
                case true:
                    doc.iloUrlsPass += contentLocation.spec + " ";
                    return passObj(obj);
                    break;
            }
        }
        //~ problems? => always return true
        return ACCEPT;
    },
    
    observe: function(aSubject, aTopic, aData)
    {
        if (aTopic == "app-startup") {   
            var prefInternal = Components.classes[PREFSERVICE_CONTRACTID].
                    getService(Components.interfaces.nsIPrefBranchInternal);
            prefInternal.addObserver("imglikeopera", this, true);
            loadSettings();
        } else {
            loadSettings();
        }
    },
    
    QueryInterface: function(iid)
    {
        if (!iid.equals(Components.interfaces.nsISupports) &&
            !iid.equals(Components.interfaces.nsISupportsWeakReference) &&
            !iid.equals(Components.interfaces.nsIContentPolicy) &&
            !iid.equals(Components.interfaces.nsIObserver))
        {
            dump("nsImgLikeOpera: QI unknown interface: " + iid + "\n");
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }
        
        return this;
    }
    
};

//~ USER filters
function checkUrlBlockByFilter(url)
{
    var policy;
    for (var i = 0; i < iloSettingsPolicyPattrens.length; i++) {
        if (iloSettingsPolicyPattrens[i][0].test(url)) {
            policy = iloSettingsPolicyPattrens[i][1];
            break;
        }
    }
    return policy;
}

//~ Dance with placeholders
function passObj(obj)
{
    obj.removeAttribute("ilo-ph-fix");
    if (obj.iloFixSH) obj.removeAttribute("height");
    if (obj.iloFixSW) obj.removeAttribute("width");
    return ACCEPT;
}

function blockObj(obj, contentLocation)
{
    //~ It'll be just "Submit Query" button
    if (obj.tagName.toLowerCase() == "input")
        return REJECT_OTHER;
    
    if (obj.getAttribute("alt") == null)
        obj.setAttribute("alt", "Image");
    
    var oW = parseInt(obj.width || obj.style.width);
    var oH = parseInt(obj.height || obj.style.height);
    
    //~ set placeholder size
    if (!oW && !oH) { obj.setAttribute("ilo-ph-fix", "tofix"); return REJECT_OTHER; }
    else if (oW && !oH) { obj.iloFixSH = true; obj.height = parseInt(oW / 0.668); }
    else if (!oW && oH) { obj.iloFixSW = true; obj.width = parseInt(oH * 0.668); }
    
    contentLocation.spec = "";
    
    obj.setAttribute("ilo-ph-fix", "fixed");
    obj.iloFixP = "2fix";
    
    obj.src = obj.src;
    
    return ACCEPT;
}

//~ check URL in cache
function urlInCache(url, docpolicy)
{
    var expire = true;
    var timenow = parseInt( (new Date().getTime()) / 1000 - 1);
    
    if (iloSettingsExpirationTime != 0)
    {
        try
        {
            var cacheEntryDescriptor = httpCacheSession.openCacheEntry(url,
                                            Components.interfaces.nsICacheEntryDescriptor,
                                            false);
            
            var expirationTime = cacheEntryDescriptor.lastModified + iloSettingsExpirationTime;
            
            if (expirationTime < timenow)
            {
                cacheEntryDescriptor.doom();
            }
            else if (cacheEntryDescriptor.expirationTime != expirationTime)
            {
                cacheEntryDescriptor.setExpirationTime(expirationTime);
                cacheEntryDescriptor.markValid();
                expire = false;
            }
            
            cacheEntryDescriptor.close();
            
            /************************************************************************
             *
             * "Я не знаю кому и зачем это нужно" (c) Вертинский
             *
             * Тут такая тема... Если повторно не обратиться сейчас к кэшу, то
             * "setExpirationTime" (см. выше) не сработает, что всплывает при
             * перезагрузке страницы.
             *
             * 2Do: Надо бы разобраться и поковырять.
             * Позже: во всяком случае, так было в FF 0.9.3
             * Ещё позже: ff1.0 -- ага, так и есть
             *
             ************************************************************************/
            
            try
            {
                cacheEntryDescriptor = httpCacheSession.openCacheEntry(url,
                                            Components.interfaces.nsICache.ACCESS_READ,
                                            false);
                if (cacheEntryDescriptor)
                    expire = false;
            }
            catch(ex) {}
            
        }
        catch(ex) {}
    }
    else if (docpolicy == 3)
    {
        try
        {
            var cacheEntryDescriptor = httpCacheSession.openCacheEntry(url,
                                            Components.interfaces.nsICache.ACCESS_READ,
                                            false);
            
            if (cacheEntryDescriptor.expirationTime > timenow)
                expire = false;
        }
        catch(ex) {}
    }
    
    return expire;
}

//~ Preferences loading.
function loadSettings()
{
    try
    {
        var prefObj = Components.classes[PREFSERVICE_CONTRACTID]
                                .getService(Components.interfaces.nsIPrefService);
        var Branch = prefObj.getBranch("imglikeopera.");
        
        iloSettingsDefaultPolicy = Branch.getIntPref("default.policy");
        iloSettingsExpirationTime = Branch.getIntPref("expiration.time");
        iloSettingsLoadFromCss = Branch.getBoolPref("css.load") ? ACCEPT : REJECT_OTHER;
        iloSettingsPolicyPattrens = iloCreatePatternsArray(Branch.getCharPref("policy.patterns"));
    }
    catch(ex) {}
}

function iloCreatePatternsArray(str)
{
    var pattern = [];
    var tmp = str.split("  ");
    var tmpCount = tmp.length-1;
    var patt;
    for (var i = 0; i < tmpCount; i++)
    {
        patt = tmp[i].split(" ");
        patt[1] = patt[1][0]-0;
        if (patt.length == 2 && patt[1]>0 && patt[1]<5)
            pattern.push( [iloTxt2RegExp(patt[0]), patt[1]] );
    }
    
    return pattern;
}

function iloTxt2RegExp(txt)
{
    //~ not simple; return power
    if (txt[0] == "/" && txt[2])
    {
        var mod = "";
        var txtEnd = txt.substr(txt.length-2, txt.length);
        
        if (txtEnd == "/i" || txtEnd == "/I") { mod = "i"; var res = txt.substr(1, txt.length-3); }
        else if (txtEnd[1] == "/") { var res = txt.substr(1, txt.length-2); }
        
        try { var reg = new RegExp(res, mod); } catch(e){}
        
        if (reg) return reg;
    }
    
    //~ else: simple;
    s = new String(txt);
    var sCount = s.length;
	res = new String("^");
    
    for (var i = 0 ; i < sCount; i++) {
        switch(s[i]) {
            case '*' :
                res += '.*';
                break;
            
            case '.' :
            case '?' :
            case '^' :
            case '$' :
            case '+' :
            case '{' :
            case '[' :
            case '|' :
            case '(' :
            case ')' :
            case ']' :
                res += '\\' + s[i];
                break;
            
            case '\\' :
                res += '\\\\';
                break;
            
            default :			
                res += s[i];
                break;
        }
    }
    
    return new RegExp(res, 'i');
}

/************************************************************************
 *
 * Module definition
 *
 ************************************************************************/
var nsImgLikeOperaModule =
{
    registerSelf: function(compMgr, fileSpec, location, type)
    {
        compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        compMgr.registerFactoryLocation(IMGLIKEOPERA_CID, 
                                                        "nsImgLikeOpera JS component", 
                                                        IMGLIKEOPERA_CONTRACTID, 
                                                        fileSpec, location, type);
                    
        var catman = Components.classes[CATMAN_CONTRACTID].getService(Components.interfaces.nsICategoryManager);
        catman.addCategoryEntry("app-startup", "ImgLikeOpera",
                                                IMGLIKEOPERA_CONTRACTID,true,true);
        catman.addCategoryEntry("content-policy", IMGLIKEOPERA_CONTRACTID,
                                                IMGLIKEOPERA_CONTRACTID,true,true);
    },
    
    unregisterSelf: function(compMgr, fileSpec, location)
    {
        compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        compMgr.unregisterFactoryLocation(IMGLIKEOPERA_CID, fileSpec);
        var catman = Components.classes[CATMAN_CONTRACTID].getService(Components.interfaces.nsICategoryManager);
        catman.deleteCategoryEntry("app-startup", IMGLIKEOPERA_CONTRACTID, true);
        catman.deleteCategoryEntry("content-policy", IMGLIKEOPERA_CONTRACTID, true);
    },
    
    getClassObject: function(compMgr, cid, iid)
    {
        if (!cid.equals(IMGLIKEOPERA_CID))
            throw Components.results.NS_ERROR_NO_INTERFACE;
        if (!iid.equals(Components.interfaces.nsIFactory))
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
        return nsImgLikeOperaFactory;
    },
    
    canUnload: function(compMgr) { return true; }
};

/**
 * Factory definition
 **/

var nsImgLikeOperaFactory =
{
    createInstance: function (outer, iid) {
    if (outer != null)
        throw Components.results.NS_ERROR_NO_AGGREGATION;
        return nsImgLikeOpera;
    }
}

/**
 * Module initialisation
 **/

function NSGetModule(comMgr,fileSpec){return nsImgLikeOperaModule;}