/***** BEGIN LICENSE BLOCK *****
    FlashGot - a Firefox extension for external download managers integration
    Copyright (C) 2004-2005 Giorgio Maone - g.maone@informaction.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

***** END LICENSE BLOCK *****/

#define VERSION "0.5.9.5"

// #define DAP

#include "stdafx.h"
#include <atlstr.h>
#include <shellapi.h>
#include <sys/stat.h>
#include <comdef.h>
#include <atlbase.h>
#include <atlcom.h>
#include <objbase.h>
#include <wininet.h>
#include <time.h>
#include <string>

using namespace std;

#define BUF_SIZE 1024

char g_buf[BUF_SIZE];
wchar_t g_wbuf[BUF_SIZE];

void fail(char *msg, int code) 
{
	MessageBox(NULL, msg,
					  "FlashGot Error",
					  MB_OK | MB_ICONERROR);
	exit(code);
}

enum OpType { 
		OP_ONE=0, OP_SEL=1, OP_ALL=2, 
		OP_QET=3,	
		OP_MIN=OP_ONE, OP_MAX=OP_QET };

typedef struct _LinkInfo 
{
	bstr_t url;
	bstr_t comment;
	bstr_t cookie;
	bstr_t postdata;
} LinkInfo;

#define EXTRAS_COUNT 5

typedef struct _DownloadInfo 
{
	char *dmName;
	OpType opType;
	bstr_t folder;
    bstr_t *rawParms;
	bstr_t referer;
	LinkInfo *links;
	int linksCount;
	bstr_t *extras;
} DownloadInfo;

class CookieManager
{
	private: 
		
		const DownloadInfo *downloadInfo;
	public:
		CookieManager() : downloadInfo(NULL) {}
		
		CookieManager(const DownloadInfo *downloadInfo) : downloadInfo(downloadInfo)
		{
			setAll(downloadInfo,300); // 5min timespan;
		}

		static void makeExpiration(char *expiration, size_t buflen, int offset)
		{
			time_t now;
			time(&now);
			now+=offset; 
			strftime(expiration, buflen,
				"%a, %d-%b-%Y %H:%M:%S GMT", gmtime(&now) );
		}

		static void setCookie(const bstr_t *url, const bstr_t *cookie, char *expiration) {
			int cookieLen;
			if(cookie  && (cookieLen=cookie->length()))
			{
				
				char *lpszUrl=*url;
				char *lpszCookie=new char[cookieLen+16+strlen(expiration)];
				
				for(char *cookiePiece=*cookie; cookiePiece && cookiePiece[0]; )
				{
					if(cookiePiece[0]==' ')
					{
						cookiePiece++;
						continue;
					}

					char *nextPiece=strchr(cookiePiece,';');
					if(nextPiece)
					{
						nextPiece[0]='\0';
						nextPiece++;
					}
					;
				
					InternetSetCookie(lpszUrl, NULL, 
						strcat(strcat(strcpy(lpszCookie,cookiePiece),"; expires = "),expiration));
					cookiePiece=nextPiece;
				}

				
				delete [] lpszCookie;
			} 
			
		}

		static setAll(const DownloadInfo *downloadInfo, long offsetExpiration)
		{
			char expiration[64];
			makeExpiration(expiration,64,offsetExpiration);
			LinkInfo *links=downloadInfo->links;
			for(int j=downloadInfo->linksCount; j-->0;)
			{
				LinkInfo l=links[j];
				setCookie(&l.url,&l.cookie,expiration);
			}
		}
	
	
};

class FGArray
{
	private:
		VARIANT varStr,array;
		SAFEARRAY *psa;
		int elemCount;
		long ix[1];
		
	
		void init(int elemCount)
		{
			this->psa=createSafeArray(elemCount);
			this->elemCount=elemCount;
			ix[0]=0;
			varStr.vt=VT_BSTR;
		}

	public:

	static SAFEARRAY *createSafeArray(int elemCount) 
		{
			SAFEARRAYBOUND bounds [1]; 
			bounds[0].cElements=elemCount;
			bounds[0].lLbound=0;
			return SafeArrayCreate(VT_VARIANT, 1, bounds);
		}


		FGArray(int elemCount)
		{
			init(elemCount);
		}
		
		FGArray(const DownloadInfo *downloadInfo)
		{		
			init(downloadInfo->linksCount * 2 + 1); // linksCount * 2 (url,info)
			addString(downloadInfo->referer);
			LinkInfo *links=downloadInfo->links;
			for (int j=0, linksCount=downloadInfo->linksCount; j < linksCount ; j++) 
			{
				LinkInfo l=links[j];
				addString(l.url);
				addString(l.comment);
			}
			
		}

		BOOL addString(bstr_t s) 
		{
			if(ix[0]<elemCount) 
			{
				varStr.bstrVal=s; 
				SafeArrayPutElement(psa, ix, &varStr); 
				ix[0]++;
				return TRUE;
			}
			return FALSE;
		}

		BOOL putString(int idx, bstr_t s)
		{
			setIdx(idx);
			return addString(s);
		}
		
		void setIdx(int idx)
		{
			ix[0]=idx;
		}
		
		VARIANT *asVariant(VARIANT *array)
		{
			array->vt = VT_ARRAY | VT_VARIANT | VT_BYREF; // VBScript Array
			array->pparray=&psa;
			return array;
		}

		VARIANT *asVariant()
		{
			return asVariant(&this->array);
		}

		~FGArray()
		{
			SafeArrayDestroy(psa);
		}
	
};


class FGCOMGuard
{
private:
	
	static FGCOMGuard *instance;
	static int refCount;

	FGCOMGuard()
	{
		CoInitialize(NULL);
	}
	~FGCOMGuard()
	{
		CoUninitialize();
	}
public:
	
	static addClient()
	{
		if(!instance) instance=new FGCOMGuard();
		refCount++;
	}
	
	static removeClient()
	{
		if(--refCount<0 || !instance) throw "FGGuard Illegal state";
		if(refCount == 0)
		{
			delete instance;
		}
		
	}


};

FGCOMGuard *FGCOMGuard::instance=NULL;
int FGCOMGuard::refCount=0;

// [START DOWNLOAD MANAGER SUPPORT CLASSES]

class DMSupport
{

protected:
	
	static BOOL createProcess(char *commandLine, PROCESS_INFORMATION *pi) {
		STARTUPINFO si;
		ZeroMemory( &si, sizeof(si) );
		si.cb = sizeof(si);
		
		PROCESS_INFORMATION *mypi=NULL;
		if(!pi) {
			pi=mypi=new PROCESS_INFORMATION;
		}
		ZeroMemory(pi, sizeof(pi) );

		
		BOOL ret=CreateProcess( NULL, 
				commandLine, // Command line. 
				NULL,             // Process handle not inheritable. 
				NULL,             // Thread handle not inheritable. 
				FALSE,            // Set handle inheritance to FALSE. 
				0,                // No creation flags. 
				NULL,             // Use parent's environment block. 
				NULL,             // Use parent's starting directory. 
				&si,              // Pointer to STARTUPINFO structure.
				pi )             // Pointer to PROCESS_INFORMATION structure.
			;
		
		if(mypi) {
			closeProcess(mypi);
			delete mypi;
		}
		
		return ret;
	}

	static void closeProcess(PROCESS_INFORMATION *pi) {
		CloseHandle( pi->hProcess );
		CloseHandle( pi->hThread );
	}


public:
	
	
	virtual void check(void) = 0;
	virtual void dispatch(const DownloadInfo *downloadInfo) = 0;
	virtual const char *getName(void) = 0;

};

#define COMCALL(Call) if(hr=FAILED(Call)) throw _com_error(hr)
#define HELPER(name) FGCOMHelper name(this->getProgId())
class FGCOMHelper
{
private:
	HRESULT hr;
	CComDispatchDriver  comObj;
	CComPtr<IDispatch>  lpTDispatch;
	
	VARIANT tmpVarStr;

	const char *className;

	
	void prepareCOMObj() {
		
		COMCALL(lpTDispatch.CoCreateInstance(_bstr_t(className)));
		comObj=lpTDispatch;
	}

public:
	
	void getMemberID(char *memberName, DISPID *dispid) {
		USES_CONVERSION;
		OLECHAR FAR* oleMember=A2OLE(memberName);
		COMCALL(comObj->GetIDsOfNames(IID_NULL, &oleMember, 1, LOCALE_SYSTEM_DEFAULT, dispid));
	}

	void invoke(DISPID *dispid, VARIANT *parms, const unsigned int parmsCount) {	
		COMCALL(comObj.InvokeN(*dispid,parms,parmsCount,NULL));
	}
	
	void invoke(DISPID *dispid) {	
		COMCALL(comObj.Invoke0(*dispid,NULL));
	}

	void invoke(char *memberName, VARIANT *parms, const unsigned int parmsCount) {
		DISPID dispid;
		getMemberID(memberName,&dispid);
		invoke(&dispid,parms,parmsCount);
	}

	void invoke(char *memberName, _bstr_t parm) {
		tmpVarStr.bstrVal=parm;
		invoke(memberName,&tmpVarStr,1);
	}
	
	void invoke(char *memberName) {
		invoke(memberName,&tmpVarStr,0);
	}

	IDispatch *getIDispatch() {
		return lpTDispatch;
	}
	void set(char *propertyName, VARIANT *val) {
		USES_CONVERSION;
		LPCOLESTR oleMember=A2COLE(propertyName);
		COMCALL(comObj.PutPropertyByName(oleMember, val));
	}
	
	void set(char *propertyName, bstr_t val) {
		tmpVarStr.bstrVal=val;
		set(propertyName,&tmpVarStr);
	}

	void get(char *propertyName, VARIANT *val, UINT indexCount) {
		DISPPARAMS dispparams = {indexCount==0?NULL:&(val[1]), NULL, indexCount, 0};
		DISPID dispid;
		getMemberID(propertyName,&dispid);
		COMCALL(lpTDispatch->Invoke(dispid, IID_NULL,
				LOCALE_USER_DEFAULT, DISPATCH_PROPERTYGET,
				&dispparams, val, NULL, NULL));
	}
	
	


	FGCOMHelper(const char *className) {
	
		this->className=className;
		prepareCOMObj();
		tmpVarStr.vt=VT_BSTR;
	}

	
	~FGCOMHelper()
	{
		
	}

};


class DMSupportCOM :
	public DMSupport
{

protected:
	
	
	virtual const char * getProgId() = 0;
	
public:
	
	DMSupportCOM() {
		FGCOMGuard::addClient();
	}
	void check() 
	{
		HRESULT hr;
		CLSID clsid;
		COMCALL(CLSIDFromProgID(_bstr_t(getProgId()),&clsid));
	}
	
	virtual ~DMSupportCOM() {
		FGCOMGuard::removeClient();
	}
	
};




#ifdef DAP

#include "DAP.hpp"

#else

class DMSAddUrlFamily :
	public DMSupportCOM
{
	void dispatch(const DownloadInfo *downloadInfo)
	{
		CookieManager cm(downloadInfo);
		HELPER(h);
		int linksCount=downloadInfo->linksCount;
		if(linksCount>0)
		{
			if(linksCount<2) {
				LinkInfo link=downloadInfo->links[0];
				VARIANT v[3];
				v[2].vt=v[1].vt=v[0].vt=VT_BSTR;
				v[2].bstrVal=link.url;
				v[1].bstrVal=link.comment;
				v[0].bstrVal=downloadInfo->referer;
				h.invoke("AddUrl",v,3);
			} 
			else 
			{
				FGArray fgArray(downloadInfo);
				h.invoke("AddUrlList",fgArray.asVariant(),1);
			}
		}
	}
};





class DMSFlashGet :
	public DMSAddUrlFamily
{

protected:
	
	const char * getProgId() { return "JetCar.Netscape"; }
	
public:
	
	const char * getName() { return "FlashGet"; }

};



class DMSFreeDownloadManager :
	public DMSupportCOM
{

protected:

	const char * getProgId() { return "WG.WGUrlListReceiver"; }

public:
	
	const char * getName() { return "Free Download Manager"; }

	void dispatch(const DownloadInfo *downloadInfo )
	{
		
		int linksCount=downloadInfo->linksCount;
		if(linksCount>0)
		{
			CookieManager cm(downloadInfo);
			const char *progId;
			char *methodName;

			if(linksCount<2)
			{
				progId="WG.WGUrlReceiver";
				methodName="AddDownload";
			} else {
				progId=getProgId();
				methodName="AddURLToList";
			}

			FGCOMHelper fdm(progId);
			fdm.set("Referer",downloadInfo->referer);
			LinkInfo *links=downloadInfo->links;
			for(int j=0; j< linksCount; j++) {
				LinkInfo l=links[j];
				fdm.set("Url",l.url);
				fdm.set("Comment",l.comment);
				fdm.invoke(methodName);
			}
			if(linksCount>1) fdm.invoke("ShowAddUrlListDialog");
			
		}
	}
	
};



class DMSFreshDownload :
	public DMSupportCOM
{


protected:
	
	const char * getProgId() { return "fdcatch.fdnscatcher"; }
	
public:
	
	const char * getName() { return "FreshDownload"; }

	void dispatch(const DownloadInfo *downloadInfo)
	{
		HELPER(h);
		CookieManager cm(downloadInfo);
		if(downloadInfo->opType==OP_ONE) 
		{
			LinkInfo l=downloadInfo->links[0];
			VARIANT v[3];
			v[2].vt=v[1].vt=VT_BSTR;
			v[2].bstrVal=l.url; // URL
			v[1].bstrVal=downloadInfo->referer; // referer
			v[0].vt=VT_INT;
			v[0].intVal=0; // quiet (0 means "show dialog")
			h.invoke("AddURL",v,3);
		} 
		else
		{
			h.invoke("AddUrlList",FGArray(downloadInfo).asVariant(),1);
		}
	}
};


class DMSGetRight :
	public DMSupport
{

private:
	static char *findGetRight(HKEY baseKey, char *leafPath, char *exeName) {
		long res;
		HKEY hk;
		char *path=NULL;
		

		if( (res=RegOpenKeyEx(baseKey,leafPath,0,KEY_QUERY_VALUE,&hk))==ERROR_SUCCESS)
		{
		
		
			char *installKey="InstallDir";
			long pathLen=0;	
			if((res=RegQueryValueEx(hk,installKey,0,NULL,NULL,(LPDWORD)&pathLen))==ERROR_SUCCESS) 
			{
				BOOL lastAttempt;
				do {
					lastAttempt=exeName==NULL;
					if(lastAttempt) exeName="getright.exe";
					
					path=new char[pathLen+strlen(exeName)+2];
					BOOL fileExists=FALSE;
					if((res=RegQueryValueEx(hk,installKey,0,NULL,(LPBYTE)path,(LPDWORD)&pathLen))==ERROR_SUCCESS)
					{
						strcat(path,"\\");
						strcat(path,exeName);
						struct stat statbuf;
						fileExists=!stat(path,&statbuf);  
					}
					if(!fileExists) {
						delete path;
						path=NULL;
						if(!lastAttempt) exeName=NULL;
					}
				} while(! (path || lastAttempt) );
			}
			RegCloseKey(hk);	
		}
		
		return path;
		
	}
	
	static char *findGetRight(char *exeName) 
	{
		char *path;
		if(	( path=findGetRight(HKEY_CURRENT_USER,"Software\\Headlight\\GetRight\\Config",exeName) )
			 || ( path=findGetRight(HKEY_LOCAL_MACHINE,"Software\\Headlight\\GetRight",exeName) ) )
		{
			return path;
		}
		throw "Can't find GetRight executable";
	}

	static char *createCmdLine(char *path, char *opts, char *arg) {
		char *cmdLine=new char[strlen(path)+strlen(opts)+strlen(arg)+6];
		sprintf(cmdLine,"\"%s\" %s %s",path,opts,arg);
		return cmdLine;
	}

public:
	
	const char * getName() { return "GetRight"; }

	void check() 
	{
		delete findGetRight(NULL);
	}
	void dispatch(const DownloadInfo *downloadInfo) 
	{
		char *cmdLine=NULL;
		char *path=NULL;
		char *exeName;
		char *arg;
		char *tgargs=NULL;
		if(downloadInfo->linksCount) {
			exeName="togetright.exe";
			if( strlen(downloadInfo->extras[2]) 
				&& (strcmp("old",downloadInfo->extras[2])==0))
			{
				arg=downloadInfo->links[0].url; // CHECK ME!!!
			} else
			{
				char *pattern="/referer=%s /cookie=\"%s\" /url=%s";
				char *cookie=downloadInfo->links[0].cookie;
				char *referer=downloadInfo->referer;
				char *url=downloadInfo->links[0].url;
				
				arg=tgargs=new char[
					strlen(referer)+
					strlen(cookie)+
					strlen(url)+
					strlen(pattern)];

				sprintf(tgargs,pattern,
					referer,cookie,url);
			}
		} else {
			exeName=NULL;
			arg=(char *)downloadInfo->referer;
		}

		BOOL ret=createProcess(cmdLine=createCmdLine(path=findGetRight(exeName),
			downloadInfo->extras[0],
			arg),NULL);
		if(tgargs) delete [] tgargs;
		// supplementary post-command
		if( ret && (!exeName) && strlen(downloadInfo->extras[1])) {
			Sleep(500);
			createProcess(cmdLine=createCmdLine(path,
			downloadInfo->extras[1],
			""),NULL);
		}
		if(cmdLine) {
			delete [] cmdLine;
		}
		if(path) delete [] path;
		if(!ret) throw "Can't launch GetRight";
	}
};

class DMSHiDownload :
	public DMSupportCOM
{


protected:
	
	const char * getProgId() { return "NetMoles.NetMoles"; }
	
public:
	
	const char * getName() { return "HiDownload"; }

	
	void dispatch(const DownloadInfo *downloadInfo)
	{
		CookieManager cm(downloadInfo);
		
		
		HELPER(h);
		
		FGArray fgArray(downloadInfo);
		VARIANT v;
		v.vt=VT_BYREF | VT_VARIANT;
		v.pvarVal=fgArray.asVariant();
		h.invoke("NMAddAllUrl",&v,1);
		
	}
};

class DMSInstantGet :
	public DMSAddUrlFamily
{
protected:
	const char * getProgId() { return "InstantGet.AddUrl"; }
public:
	const char * getName() { return "InstantGet"; }
};


class DMSLeechGetBase :
	public DMSupportCOM
{

public:
	
	
	void dispatch(const DownloadInfo *downloadInfo) 
	{	
		HELPER(h);
		CookieManager cm(downloadInfo);
		if(downloadInfo->linksCount>0) 
		{
			h.invoke(downloadInfo->opType==OP_ONE?"AddURL":"Wizard", downloadInfo->links[0].url);
		} 
		else 
		{
			h.invoke("Parse",downloadInfo->referer);
		}
	}
};

class DMSLeechGet2002 :
	public DMSLeechGetBase
{
	
protected:
	const char * getProgId() 
	{
		return "LeechGetIE.AddURL";
	}
public:
	const char * getName() { return "LeechGet 2002"; }
};

class DMSLeechGet :
	public DMSLeechGetBase
{
protected:
	const char * getProgId() 
	{
		return "LeechGetIE.LeechIE";
	}
public:
	const char * getName() { return "LeechGet"; }
};

class DMSMass_Downloader :
	public DMSupportCOM
{


protected:
	
	const char * getProgId() { return "MassDown.AddUrl.1"; }
	
public:
	
	const char * getName() { return "Mass Downloader"; }

	void dispatch(const DownloadInfo *downloadInfo)
	{
		
		
		int linksCount=downloadInfo->linksCount;
		if(linksCount<1) return;
		{
			VARIANT v[3];
			v[2].vt=v[1].vt=v[0].vt=VT_BSTR;
			HELPER(h);
			CookieManager cg(downloadInfo);
			DISPID di_AddUrlWithReferer;
			h.getMemberID("AddUrlWithReferer",&di_AddUrlWithReferer);
			
			if(linksCount>1) {
				v[1].bstrVal=v[0].bstrVal=bstr_t("Begin.");
				h.invoke("AddUrl",v,2);
			}
			v[0].bstrVal=downloadInfo->referer;
			LinkInfo *links=downloadInfo->links;
			for(int j=0; j<linksCount; j++) {
				LinkInfo l=links[j];
				v[2].bstrVal=l.url;
				v[1].bstrVal=l.comment;
				h.invoke(&di_AddUrlWithReferer,v,3);
			}
			if(linksCount>1) {
				v[1].bstrVal=v[0].bstrVal=bstr_t("End.");
				h.invoke("AddUrl",v,2);
			}
		}
	}
};

class DMSNetAnts :
	public DMSAddUrlFamily
{
protected:
	const char * getProgId() { return "NetAnts.API"; }
public:
	const char * getName() { return "NetAnts"; }
};


class DMSNet_Transport :
	public DMSupportCOM
{

protected:
	
	const char * getProgId() { return "NTIEHelper.NTIEAddUrl"; }

public:
	const char * getName() { return "Net Transport"; }

	void dispatch(const DownloadInfo *downloadInfo)
	{
		CookieManager cm(downloadInfo);
		
		int linksCount=downloadInfo->linksCount;
		bstr_t *parms=downloadInfo->rawParms;
		
		SAFEARRAY *psaa[2]=
		{
			FGArray::createSafeArray(linksCount),
			FGArray::createSafeArray(linksCount)
		};

		VARIANT vstr;
		vstr.vt = VT_BSTR;
		long ix[]={0};
		for(int j=1; ix[0]<linksCount; ix[0]++) 
		{
			for(int a=0; a<2; a++) 
			{
				vstr.bstrVal=parms[j++];
				SafeArrayPutElement(psaa[a], ix, &vstr);
			}
			j+=2; // skipping cookie & postData;
		}
		
		//AddList(referrer,urls, remarks);
		VARIANT v[3];
		v[2].vt=VT_BSTR;
		v[2].bstrVal=downloadInfo->referer;
		v[1].vt=v[0].vt=VT_VARIANT | VT_BYREF ;
		SafeArrayAccessData(psaa[0], (void **)&(v[1].pvarVal));
		SafeArrayAccessData(psaa[1], (void **)&(v[0].pparray));
		HELPER(h);
		h.invoke("AddList",v,3);

		SafeArrayUnlock(psaa[1]);
		SafeArrayUnlock(psaa[0]);
		SafeArrayDestroy(psaa[0]);
		SafeArrayDestroy(psaa[1]);
	}
};

class DMSWestByte :
	public DMSupportCOM
{

public:


	void dispatch(const DownloadInfo *downloadInfo)
	{
		int linksCount=downloadInfo->linksCount;
		bstr_t *parms=downloadInfo->rawParms;
		
		if(linksCount>0)
		{
			
			VARIANT v[2];
			v[0].vt = VT_BSTR ;
			v[0].bstrVal=downloadInfo->referer; // referer
			HELPER(h);
			CookieManager cm(downloadInfo);
			if(linksCount==1) 
			{
				v[1].vt = VT_BSTR; // VBScript Array
				v[1].bstrVal=downloadInfo->links[0].url;
				
				h.invoke("AddURL",v,2);
			} 
			else 
			{ 

				FGArray fgArray(linksCount * 2);
				LinkInfo *links=downloadInfo->links;
				for (int j=0; j < linksCount; j++) {
					LinkInfo l=links[j];
					fgArray.addString(l.url);
					fgArray.addString(l.comment);
				}
				
				fgArray.asVariant(&v[1]);
				
				h.invoke("AddURLs",v,2);

			}
		}
	}
	
};

class DMSDownloadMaster :
	public DMSWestByte
{
protected:
	const char * getProgId() { return "DMIE.MoveURL"; }
public:
	const char * getName() { return "Download Master"; }
};

class DMSInternetDownloadAccelerator :
	public DMSWestByte
{
protected:
	const char * getProgId() { return "IDAIE.MoveURLIDA"; }
public:
	const char * getName() { return "Internet Download Accelerator"; }
};








class DMSReGet :
	public DMSupportCOM
{
	
	protected:
	const char * getProgId() 
	{
		return "ClickCatcher.DownloadAllFromContextMenu";
	}
public:
	const char * getName() { return "ReGet"; }
	
	void dispatch(const DownloadInfo *downloadInfo) 
	{	
		int linksCount=downloadInfo->linksCount;
		
		VARIANT v[5];
		v[0].vt=v[1].vt=v[2].vt=v[3].vt=v[4].vt=VT_BSTR;
		bstr_t referer=downloadInfo->referer; // referer
		
		FGCOMHelper *regetAll=NULL;
		DISPID di_AddDownload;
	
		if(linksCount>1) {
			regetAll=new FGCOMHelper(getProgId());
			regetAll->getMemberID("AddDownloadToList",&di_AddDownload);
		} else if(linksCount==1) {
			LinkInfo l=downloadInfo->links[0];
			if(strlen(l.postdata)) {
				try 
				{
					FGCOMHelper dl("ReGetDx.RegetDownloadApi");
					dl.set("Url",l.url);
					dl.set("Referer",referer);
					dl.set("Cookie",l.cookie);
					dl.set("Info",l.comment);
					dl.set("PostData",l.postdata);
					dl.invoke("AddDownload");
					return;
				} catch(...) {}
			}
		}
		CookieManager cm(downloadInfo);
		LinkInfo *links=downloadInfo->links;
		for(int j=0; j<linksCount; j++) 
		{	
			LinkInfo l=links[j];
			
			for(int attempts=120; attempts-->0;) 
			{ // we give ReGet 2mins to wake up
				try 
				{
					FGCOMHelper dl("ClickCatcher.DownloadFromContextMenu");
					
					dl.set("Url",l.url);
					dl.set("Referer",referer);
					dl.set("Info",l.comment);
					dl.set("Cookie",l.cookie); // -- it seems a dummy property, we use CookieManager which works
					if(regetAll) {
						VARIANT vdl;
						vdl.vt=VT_DISPATCH;
						vdl.pdispVal=dl.getIDispatch();
						regetAll->invoke(&di_AddDownload,&vdl,1);
					} else {
						dl.invoke("AddDownload");
					}

					break;
				} 
				catch(_com_error ce)
				{
					Sleep(1000);
					if(attempts==0) throw "ReGet timeout";
				}
			}
		}

		if(regetAll) try 
		{ 
			regetAll->invoke("UrlList",NULL,0);
			delete regetAll;
		} catch(...) {}
	}
};


class DMSStarDownloader :
	public DMSupportCOM
{


protected:
	
	const char * getProgId() { return "SDExt.StarDownExt"; }
	
public:
	
	const char * getName() { return "Star Downloader"; }

	void dispatch(const DownloadInfo *downloadInfo)
	{
		
		HELPER(downloader);
		DISPID di_DownloadURL;
		downloader.getMemberID("DownloadURL",&di_DownloadURL);
		
		VARIANT v[2];
		v[0].vt = v[1].vt = VT_BSTR;
		v[0].bstrVal = downloadInfo->referer; // referrer
		LinkInfo *links=downloadInfo->links;
		for (long j=0, len=downloadInfo->linksCount; j < len ; j++) { 
			v[1].bstrVal=links[j].url;
			downloader.invoke(&di_DownloadURL,v,2);
		}
	}
};

class DMSThunder :
	public DMSupportCOM
{


protected:

	const char * getProgId() { return "Xunleibho.CatchRightClick.1"; }
	
public:
	
	const char * getName() { return "Thunder"; }

	void dispatch(const DownloadInfo *downloadInfo)
	{
		CookieManager cm(downloadInfo);
		HELPER(h);
		VARIANT v[2];
		v[0].vt= VT_BYREF;
		if(downloadInfo->opType==OP_ONE) 
		{
			LinkInfo l=downloadInfo->links[0];
			v[1].vt=VT_BSTR;
			v[1].bstrVal="#*01#*" + l.url + "#*02#*" + downloadInfo->referer + "#*03#*" + l.comment + "#*04#*Ѹ4#*05#*";
			h.get("sendUrl",v,1);
		} 
		else
		{
			FGArray fgArray(downloadInfo);
			v[1].vt=VT_BYREF | VT_VARIANT;
			v[1].pvarVal=fgArray.asVariant();
			h.get("AddAllUrl",v,1);
		}
	}
};


class DMSTrueDownloader :
	public DMSupportCOM
{


protected:
	
	const char * getProgId() { return "TrueDownloaderProject.TrueDownloader"; }
	
public:
	
	const char * getName() { return "TrueDownloader"; }

	void dispatch(const DownloadInfo *downloadInfo)
	{
		
		HELPER(downloader);
		DISPID di_MenuURL;
		downloader.getMemberID("MenuURL",&di_MenuURL);
		
		VARIANT v[3];
		v[0].vt = v[1].vt = v[2].vt = VT_BSTR;
		v[1].bstrVal = downloadInfo->referer; // referrer
		LinkInfo *links=downloadInfo->links;
		for (long j=0, len=downloadInfo->linksCount; j < len ; j++) 
		{ 
			v[2].bstrVal=links[j].url;
			v[0].bstrVal=links[j].cookie;
			downloader.invoke(&di_MenuURL,v,3);
		}
	}
};

class DMSWellGet :
	public DMSupport
{
	

public:
	
	const char * getName() { return "WellGet"; }
	
	void check() {
		dispatch(NULL);
	}

	void dispatch(const DownloadInfo *downloadInfo)
	{
		
	
		HKEY hk = NULL;
		DWORD dwDisposition = NULL;
		BOOL ok=false;
		if(RegOpenKeyEx(HKEY_CURRENT_USER,_T("Software\\WellGet"),0,KEY_QUERY_VALUE,&hk)==ERROR_SUCCESS)
		{
			dwDisposition = 1023;
			BYTE szpath[1024];
			DWORD type_1=REG_SZ ; 
			if(RegQueryValueEx(hk, NULL, NULL,&type_1, szpath, &dwDisposition)==ERROR_SUCCESS)
			{
				CString apppath;
				apppath.Format("%s\\WellGet.exe",szpath);
				{
					struct stat statbuf;
					if(!stat(apppath,&statbuf)) ok=true;
				}
				if(ok && downloadInfo && downloadInfo->linksCount>0)
				{
					CookieManager cm(downloadInfo);
					CString parms;

					if(downloadInfo->opType==OP_ONE) 
					{
						LinkInfo l=downloadInfo->links[0];
						
						parms.Format("-r:\"%s\" -c:\"%s\" \"%s\"",
							(char *)downloadInfo->referer,
							(char *)l.comment,(char *)l.url);
					} 
					else 
					{
						LinkInfo *links=downloadInfo->links;
						
						FILE * fp; 
						char path[1024];
						if (GetTempPath(1024,path)==0)
						{
							strcpy(path,"c:");
						}
						strcat(path,"TEMPURL.TXT");

						if(!(fp = fopen(path, "w"))) 
						{ 
							throw "FlasGot can't create file TEMPURL.TXT for WellGet";
						} 
						fputs(downloadInfo->referer,fp);
						fputc('\n',fp);
						for(int j=0,count=downloadInfo->linksCount; j<count; j++) {
							LinkInfo l=links[j];
							fputs(l.url,fp);
							fputc('\n',fp);
							fputs(l.comment,fp);
							fputc('\n',fp);
						}
						fclose(fp);
						parms="-u addlist";
					}
					ShellExecute(NULL,"open",
						apppath,parms,"", SW_SHOW );
				}
				
			}
			RegCloseKey(hk);
		}
		
		if(!ok) throw "Can't find WellGet executable";
	}
};


#endif

// [END DOWNLOAD MANAGER SUPPORT CLASSES]


typedef struct _DMSNode {
	unsigned int id;
	DMSupport *dms;
	_DMSNode *prev;
} DMSNode;



class DMSFactory 
{
private:
	static DMSFactory *instance;
	DMSNode *last;
	DMSupport *add(DMSupport *dms) {
		if(!last) {
			(last=new DMSNode())->prev=NULL;
			last->id=1;
		} else {
			DMSNode *newNode=new DMSNode();
			newNode->prev=last;
			newNode->id=last->id << 1;
			last=newNode;
		}
		return last->dms=dms;
	}

	DMSFactory() : last(NULL) {
#ifdef DAP
		add(new DMSDownloadAcceleratorPlus());
#else
		add(new DMSDownloadMaster());
		add(new DMSFlashGet());
		add(new DMSFreeDownloadManager());
		add(new DMSFreshDownload());
		add(new DMSGetRight());
		add(new DMSHiDownload());
		add(new DMSInstantGet());
		add(new DMSInternetDownloadAccelerator());
		add(new DMSLeechGet2002());
		add(new DMSLeechGet());
		add(new DMSMass_Downloader());
		add(new DMSNetAnts());
		add(new DMSNet_Transport());
		add(new DMSReGet());
		add(new DMSStarDownloader());
		add(new DMSTrueDownloader());
		add(new DMSThunder()); 
		add(new DMSWellGet());
#endif
}

public:
	
	static DMSFactory *getInstance() {
		return instance?instance:instance=new DMSFactory();		
	}
	
	
	DMSupport *getDMS(char *name) {
		DMSNode *cursor=last;
		for(; cursor && strcmp(cursor->dms->getName(),name); cursor=cursor->prev);
		return cursor 
			?cursor->dms
			:NULL;	
	}
	
	unsigned int checkAll() {
		unsigned int retVal=0;
		fprintf(stdout,"FlashGot Win Bridge %s\r\n",VERSION);
		FGCOMGuard::addClient();
		for(DMSNode *cursor=last; cursor; cursor=cursor->prev) {
			try {
				fprintf(stdout,"%s|",cursor->dms->getName());
				cursor->dms->check();
				fprintf(stdout,"OK\n");
				continue;
			} catch(_com_error ce) {
				fprintf(stdout,"BAD\nCOM error: %s\n", ce.ErrorMessage());
			} catch(...) {
				fprintf(stdout,"BAD\nunexpected unknown error!\n");
			}
			retVal |= cursor->id;
		}
		FGCOMGuard::removeClient();
		return retVal;
	}



	~DMSFactory() {
		DMSNode *cursor=last;
		while( cursor ) {
			delete cursor->dms;
			last=cursor;
			cursor=last->prev;
			delete last;
		}
	}
};
DMSFactory * DMSFactory::instance=NULL;




char* readLine(FILE *stream, char *buf) 
{
	
	char *res=fgets(buf,BUF_SIZE,stream);
	if(res) 
	{
		size_t lastPos=strlen(res)-1;
		while(lastPos>=0 && 
			(res[lastPos]==0x0a || res[lastPos]==0x0d)
			) res[lastPos--]='\0';
	}
	return res;
}





DMSupport* createDMS(char *name) {
	DMSupport *res=DMSFactory::getInstance()->getDMS(name);
	if(res) return res;
	sprintf(g_buf,"Unsupported Download Manager %s", name);
	fail(g_buf,-8000);
	return NULL;
}

wchar_t *UTF8toUnicode(char *buf) {
	return buf && (
		MultiByteToWideChar(CP_UTF8,0,buf,-1,g_wbuf,BUF_SIZE) 
		|| MultiByteToWideChar(CP_ACP,0,buf,-1,g_wbuf,BUF_SIZE))
		? g_wbuf : L""
		;
}




int performTest(char *outfname)
{
	if(outfname) 
	{
		freopen(outfname,"w", stdout);
		freopen(outfname,"a", stderr);
	}
	exit(DMSFactory::getInstance()->checkAll());
}


void parseHeader(DownloadInfo *downloadInfo, char *header_buf)
{
#define HEADER_COUNT 4
	char *header[HEADER_COUNT];
	char *cur=header_buf;
	for(int j=0; j<HEADER_COUNT ; ) 
	{
		if( (header[j++]=cur)  && (cur=strchr(cur,';')) ) 
		{
			cur[0]='\0';
			++cur;
		} 
		else
		{
			if(j!=HEADER_COUNT) fail("Malformed header",17);	
			break;
		}
	}

	downloadInfo->linksCount=atoi(header[0]);
	downloadInfo->dmName=header[1];
	int typeId=atoi(header[2]);
	downloadInfo->opType = (typeId<OP_MIN||typeId>OP_MAX)
		?OP_ALL:(OpType)typeId;
	downloadInfo->folder = bstr_t(UTF8toUnicode(header[3]));
}

void processJobFile(FILE *f)
{
	DMSupport *dms = NULL;
	const char *errMsg="Download manager not properly installed.\n%s";
	BOOL completed=FALSE;
	try 
	{
		DownloadInfo downloadInfo;

		char header_buf[BUF_SIZE];
		parseHeader(&downloadInfo,readLine(f,header_buf));
		int linksCount=downloadInfo.linksCount;
		int parmsCount=1 + linksCount*4 + EXTRAS_COUNT; // referer + (url + info  + cookie + postdata) * 4 + referer cookie + referer referer :-)

		bstr_t *parms=downloadInfo.rawParms=new bstr_t[parmsCount];
		
		LinkInfo *links=downloadInfo.links=new LinkInfo[linksCount];
		
		for(int j=0; j<parmsCount; j++) {
			parms[j]=bstr_t(UTF8toUnicode(readLine(f,g_buf)));
		}
		
		downloadInfo.referer=parms[0];
		downloadInfo.links=(LinkInfo *) &parms[1];
		downloadInfo.extras=&parms[parmsCount-EXTRAS_COUNT];

		(dms=createDMS(downloadInfo.dmName))->dispatch(&downloadInfo);
		printf("%s",downloadInfo.folder);
		completed=TRUE;
	} 
	catch(_com_error ce)
	{
		sprintf(g_buf,errMsg,ce.ErrorMessage());
	}
	catch(char *ex) {
		sprintf(g_buf,errMsg,ex);
	} 
	catch(...) {
		sprintf(g_buf,errMsg,"");
	}
	
	if(dms) delete dms;
	
	if(!completed) fail(g_buf,0x02000);
}

int performDownload(char *fname)
{
	FILE *f;
	
	char *done=strcat(strcpy(new char[(strlen(fname)+6)],fname),".done");
	
	if(!(f=fopen(fname,"rb")))  
	{
		struct stat statbuf;
		if(stat(done,&statbuf)) // done file doesn't exist  
		{
			sprintf(g_buf,"Can't open file %s",fname);
			fail(g_buf,0x01000);
		} else {
			remove(done);
		}
	}
	else
	{	
		if(!feof(f))	
		{
			processJobFile(f);
		}
		else 
		{
			sprintf(g_buf,"Temporary file %s is empty",fname);
			fail(g_buf,0x03000);
		}
		fclose(f);
		
		if(!strstr(done,"test"))
		{
			remove(done);
			rename(fname,done);
		}
	}
	delete [] done;
	return 0;
}

int main(int argc, char* argv[])
{
	return (argc<2 || strcmp(argv[1],"-o") == 0 )
		?performTest(argc>2?argv[2]:NULL)
	    :performDownload(argv[1]);
}

int WINAPI
WinMain(HINSTANCE inst, HINSTANCE previnst, LPSTR cmdline, int cmdshow)
{
    return main(__argc, __argv); 
}
