/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Forecastfox.
 *
 * The Initial Developer of the Original Code is
 * Jon Stritar <jstritar@MIT.EDU>.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Jon Stritar <jstritar@MIT.EDU>
 * Richard Klein <richwklein@mchsi.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var gOptions = null;

function optionsLoad()
{
  gOptions = new OptionsModule();
  gOptions.start();
}

function optionsUnload()
{
  gOptions.stop();
  gOptions = null;
}

function OptionsModule()
{
  this.wrappedJSObject = this;
  return this;
}

//XXX simplify code
OptionsModule.prototype = {
  _manager: null,
  _guid: null,
  mAlert: null,
  mBranch: null,
  mPositionMax: null,
  mErrors: null,
  
  start: function()
  {
    this._manager = Components.classes["@ensolis.com/forecastfox/manager;1"].getService(Components.interfaces.ffIManager);
    var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService); 
    this.mBranch = prefService.getBranch("forecastfox.");
    this.mErrors = new Array();
    
    try {
      var app = Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULAppInfo);    
      this._guid = app.ID;
    } catch(e) {
      var appBranch = prefService.getBranch(null);
      try {
        this._guid = appBranch.getCharPref("app.id");
      } catch(e) {
        this._guid = "{86c18b42-e466-45a9-ae7a-9b95ba6f5640}";
      };    
    };
        
    //stop rotating profiles
    this._manager.profiles.rotate(this._manager.profiles.ROTATE_STOP);

    //add observers      
    var obs = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);  
    obs.addObserver(this, "forecastfox:manager", false); 
    
    // Set up the display for the prefs
    this.populateProfiles();
    this.populateOptions();
  
    //enable-disable slider options
    try {
      this.mAlert = Components.classes["@mozilla.org/alerts-service;1"].getService(Components.interfaces.nsIAlertsService);
    } catch(e) {
      document.getElementById("cc-slider").setAttribute("hidden", true); 
      document.getElementById("swa-slider").setAttribute("hidden", true);         
    }
    
    if (this.mBranch.getCharPref("general.locid") == "00000") {
      var locid = document.getElementById("ff-text-code");
      locid.focus();
      locid.select();
    }
    
    // set the validate labels
    this.populateValidLabels();

    //select general page
    var pages = document.getElementById("ff-list-pages");
    pages.selectedItem = pages.childNodes[0];     
  },
  
  stop: function()
  {
    //start rotating profiles
    this._manager.profiles.rotate(this._manager.profiles.ROTATE_START);
 
    //remove observers
    try {
      var obs = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
      obs.removeObserver(this, "forecastfox:manager");
      obs = null;
    } catch(e) {};
    
    //release references
    this._manager = null;
    this.mAlert = null;
    this.mBranch = null;
    this.mPositionMax = null;
    this.mErrors = null;
    this._guid = null;
  },
  
  switchPage: function(aList)
  {
    var deck = document.getElementById("ff-deck-pages");
    var page = aList.selectedIndex;
    deck.selectedIndex = page;  
  },
  
  populateProfiles: function()
  {
    var list = document.getElementById("ff-list-profs");
    var popup = document.getElementById("ff-popup-profs");    
    var current = this.mBranch.getCharPref("profile.current");
    var count = {};
    var names = this._manager.profiles.getProfiles(count);
    var item;

    //remove old profiles
    while (popup.hasChildNodes())
      popup.removeChild(popup.lastChild);
    
    //append new profiles     
    for (var x = 0; x < count.value; x++) {
      item = document.createElement("menuitem");
      item.setAttribute("id", names[x]);
      item.setAttribute("label", names[x]);
      item.setAttribute("type", "prof");
    
      if (names[x] == current)
        list.selectedItem = item;
      
      popup.appendChild(item);
    }
    
    // if the current profile isn't around, select the first
    // one in the list, and switch profiles to it.
    if (list.selectedItem == null) {
      list.selectedItem = popup.firstChild;
      this.mBranch.setCharPref("profile.current", list.selectedItem.getAttribute("id"));
    }
  },
  
  populateOptions: function()
  {
    //general
    this.setElement("ff-text-code", "general.locid", "Char", null);
    this.setElement("ff-group-units", "general.units", "Int", "units");
    this.populateBars();
    this.populatePositions();
  
    //swa panel
    this.setElement("ff-chk-swa-panel", "swa.panel.enabled", "Bool", null);
    this.setElement("ff-list-swa-panel-display", "swa.panel.display", "Int", "ff-popup-swa-panel-display");
  
    //swa tooltip 
    this.setElement("ff-chk-swa-tooltip", "swa.tooltip.enabled", "Bool", null);
    this.setElement("ff-list-swa-tooltip-display", "swa.tooltip.display", "Int", "ff-popup-swa-tooltip-display");   
  
    //swa slider
    this.setElement("ff-chk-swa-slider", "swa.slider.enabled", "Bool", null);
    this.setElement("ff-list-swa-slider-display", "swa.slider.display", "Int", "ff-popup-swa-slider-display");
  
    //radar panel
    this.setElement("ff-chk-radar-panel", "radar.panel.enabled", "Bool", null);
    this.setElement("ff-list-radar-panel-display", "radar.panel.display", "Int", "ff-popup-radar-panel-display");
  
    //radar tooltip 
    this.setElement("ff-chk-radar-tooltip", "radar.tooltip.enabled", "Bool", null);
    this.setElement("ff-list-radar-tooltip-display", "radar.tooltip.display", "Int", "ff-popup-radar-tooltip-display");   
        
    //current panel
    this.setElement("ff-chk-cc-panel", "cc.panel.enabled", "Bool", null);
    this.setElement("ff-list-cc-panel-display", "cc.panel.display", "Int", "ff-popup-cc-panel-display");
  
    //current tooltip 
    this.setElement("ff-chk-cc-tooltip", "cc.tooltip.enabled", "Bool", null);
    this.setElement("ff-list-cc-tooltip-display", "cc.tooltip.display", "Int", "ff-popup-cc-tooltip-display");   
  
    //current slider
    this.setElement("ff-chk-cc-slider", "cc.slider.enabled", "Bool", null);
    this.setElement("ff-list-cc-slider-display", "cc.slider.display", "Int", "ff-popup-cc-slider-display");
    this.setElement("ff-text-cc-slider", "cc.slider.freq", "Int", null);
  
    //today's panel
    this.setElement("ff-chk-dayt-panel", "dayt.panel.enabled", "Bool", null);  
    this.setElement("ff-list-dayt-panel-display", "dayt.panel.display", "Int", "ff-popup-dayt-panel-display");
    this.setElement("ff-list-dayt-panel-mode", "dayt.panel.mode", "Int", "ff-popup-dayt-panel-mode");
   
    //today's tooltip
    this.setElement("ff-chk-dayt-tooltip", "dayt.tooltip.enabled", "Bool", null);
    this.setElement("ff-list-dayt-tooltip-display", "dayt.tooltip.display", "Int", "ff-popup-dayt-tooltip-display");       
  
    //today's switch 
    this.setElement("ff-list-dayt-switch", "dayt.panel.switch", "Int", "ff-popup-dayt-switch");
    
    //extended panel
    this.setElement("ff-text-dayf-days", "dayf.panel.days", "Int", null); 
    this.setElement("ff-chk-dayf-panel", "dayf.panel.enabled", "Bool", null); 
    this.setElement("ff-list-dayf-panel-display", "dayf.panel.display", "Int", "ff-popup-dayf-panel-display"); 
    this.setElement("ff-list-dayf-panel-mode", "dayf.panel.mode", "Int", "ff-popup-dayf-panel-mode");             
  
    //extended tooltip
    this.setElement("ff-chk-dayf-tooltip", "dayf.tooltip.enabled", "Bool", null);
    this.setElement("ff-list-dayf-tooltip-display", "dayf.tooltip.display", "Int", "ff-popup-dayt-tooltip-display");       

    //profiles
    this.setElement("ff-chk-profs-rotate", "profile.switch.enabled", "Bool", null);
    this.setElement("ff-text-profs-rotate", "profile.switch.delay", "Int", null);
  },

  populateBars: function()
  {
    // Creates the menuitems for the toolbar selector.
    var win = topWindow();
    var toolbars = win.document.getElementsByTagName("toolbar");
    var statusbars = win.document.getElementsByTagName("statusbar");
    var menubars = win.document.getElementsByTagName("menubar");
    var popup = document.getElementById("ff-popup-bars");
    var x, bar, item, val, list;
    
    // first remove the toolbars already there...
    while (popup.hasChildNodes())
      popup.removeChild(popup.firstChild);
          
    toolbars = concat(toolbars, statusbars);
    toolbars = concat(toolbars, menubars);
    for (x=0; x<toolbars.length; x++) {
      bar = toolbars[x];
      
      //do not include find toolbar
      if (bar.getAttribute("id") == "FindToolbar")
        continue;
             
      item = document.createElement("menuitem");
      item.setAttribute("id", bar.getAttribute("id"));
      item.value = bar.getAttribute("id");         
      if (bar.hasAttribute("toolbarname"))
        item.setAttribute("label", bar.getAttribute("toolbarname"));
      else
        item.setAttribute("label", bar.getAttribute("id"));                 
      popup.appendChild(item);
    }
    
    this.setElement("ff-list-bars", "general.bar", "Char", "ff-popup-bars");    
  },

  populatePositions: function()
  {
    var barid = document.getElementById("ff-list-bars").selectedItem.getAttribute("id");
    var win = topWindow();
    var bar = win.document.getElementById(barid);

    if (!bar)
      this.mPositionMax = 0;
    else {
      var len = bar.childNodes.length;
    
      //don't include ourself in count
      if (win.document.getElementById("ff-box").parentNode == bar)
        len--;
    
      this.mPositionMax = len;
        
    }

    var val = this.mBranch.getIntPref("general.position");
    var val2 = this.mBranch.getCharPref("general.bar");
    var text = document.getElementById("ff-text-position");
    var radio = document.getElementsByAttribute("group", "position");
    var group = document.getElementById("ff-group-position");
    var x, el;
    
    //set max value
    text.value = this.mPositionMax;
    
    //mark always last
    if (val == -1) {
      for (x=0; x < radio.length; x++) {
        if (parseInt(radio[x].value) == val) {
          group.selectedItem = radio[x];
          break;
        }
      }
    }
    
    //mark specific position
    else {
      for (x=0; x < radio.length; x++) {
        if (parseInt(radio[x].value) != -1) {
          group.selectedItem = radio[x];
          break;
        }
      }    
      if (barid != val2) {
        text.value = this.mPositionMax;
      } else {
        if (val < 0)
          text.value = this.mPositionMax;
        else if (val > this.mPositionMax)
          text.value = 0;
        else
          text.value = val; 
      }
    }
    
    // update the error label
    el = document.getElementById("ff-text-position-valid");
    el.setAttribute("value", "0-"+this.mPositionMax);
    
    //revalidate position
    setTimeout("gOptions.validate()", 5);
  },
  
  populateValidLabels: function() 
  {  
    var el;
    
    el = document.getElementById("ff-text-position-valid");
    el.setAttribute("value", "0-"+this.mPositionMax);
    
    el = document.getElementById("ff-text-cc-slider-valid");
    el.setAttribute("value", "0-16");
    
    el = document.getElementById("ff-text-dayf-days-valid");
    el.setAttribute("value", "0-9");
    
    el = document.getElementById("ff-text-profs-rotate-valid");
    el.setAttribute("value", "0-480");
  },
  
  setElement: function(aName, aPref, aType, aGroup)
  {
    var x, el, els, val;
    
    el = document.getElementById(aName);
    val = eval("this.mBranch.get" + aType + "Pref('" + aPref + "')");
    switch (el.localName) {
      case "checkbox":
        el.checked = val;
        break;
      case "textbox":
        el.value = val;
        break;
      case "menulist":
        els = document.getElementById(aGroup).childNodes;        
        for (x=0; x < els.length; x++) {
          if (((aType == "Int") ? parseInt(els[x].value) : els[x].value) == val) {
            el.selectedItem = els[x];
            break;
          }
        }      
        break;            
      case "radiogroup":
        els = document.getElementsByAttribute("group", aGroup);        
        for (x=0; x < els.length; x++) {
          if (((aType == "Int") ? parseInt(els[x].value) : els[x].value) == val) {
            el.selectedItem = els[x];
            break;
          }
        }      
        break;
    }  
  },
  
  setOption: function(aName, aPref, aType)
  {
    var el = document.getElementById(aName);
    switch (el.localName) {
      case "checkbox":
        eval("this.mBranch.set" + aType + "Pref('" + aPref + "'," + el.checked + ")");
        break;
      case "textbox":
        eval("this.mBranch.set" + aType + "Pref('" + aPref + "'," + ((aType == "Int") ? parseInt(el.value) : '"' + this._encodeValue(el.value) + '"') + ")");
        break;
      case "menulist":
      case "radiogroup":
        eval("this.mBranch.set" + aType + "Pref('" + aPref + "'," + ((aType == "Int") ? parseInt(el.selectedItem.value) : "'" + el.selectedItem.value + "'") + ")");
        break;
    }
  },
  
  selectProfile: function(aList)
  {
    var item = aList.selectedItem.getAttribute("id");
    var popup = document.getElementById("ff-popup-profs");
    popup.hidePopup();
  
    this.mBranch.setCharPref("profile.current", item);
  },
  
  validate: function()
  {
    //remove error indicators
    for (var i=0; i < this.mErrors.length; i++)
      this.mErrors[i].removeAttribute("error");
    this.mErrors = new Array();
    
    //check location code
    var el = document.getElementById("ff-text-code");
    if (el.value == "")
      return this.markError(el, 0);
    
    //check position
    el = document.getElementById("ff-text-position");
    var el2 = document.getElementById("ff-text-position-valid");
    if (isNaN(parseInt(el.value)) || el.value < 0 || el.value > this.mPositionMax)
      return this.markError(el, 0, el2);
    
    //check slider
    el = document.getElementById("ff-text-cc-slider");
    el2 = document.getElementById("ff-text-cc-slider-valid");
    if (isNaN(parseInt(el.value)) || el.value < 0 || el.value > 16)
      return this.markError(el, 3, el2);
    
    //check extended days
    el = document.getElementById("ff-text-dayf-days");
    el2 = document.getElementById("ff-text-dayf-days-valid");
    if (isNaN(parseInt(el.value)) || el.value < 0 || el.value > 9)
      return this.markError(el, 5, el2);
      
    //check profiles rotat
    el = document.getElementById("ff-text-profs-rotate");
    el2 = document.getElementById("ff-text-profs-rotate-valid");
    if (isNaN(parseInt(el.value)) || el.value < 0 || el.value > 480)
      return this.markError(el, 6, el2);      

    //return valid      
    return true;
  },

  markError: function(aEl, aIndex, aE2)
  {
    // aE2 is optional: xul object containing a description that will be shown
    //get page list  
    var list = document.getElementById("ff-list-pages");
    
    //mark element
    aEl.setAttribute("error", "true");
    if (aE2)
      aE2.setAttribute("error", "valid");
      // set to valid so css rules don't interfere
    
    //switch to page
    list.selectedItem = list.childNodes[aIndex];     
    this.switchPage(list);
    
    //add to error array
    this.mErrors.push(aEl);
    if (aE2)
      this.mErrors.push(aE2);
    
    return false;
  },
  
  accept: function(aClose)
  {
    if (!this.validate())
      return;

    this.updateOptions();
    
    //make sure we save profile
    this._manager.profiles.save(this._manager.profiles.current);
    
    if (aClose)  
      window.close();
    else
      document.getElementById("apply").setAttribute("disabled", "true");
  },

  updateOptions: function()
  {
    //general
    this.setOption("ff-text-code", "general.locid", "Char");
    this.setOption("ff-group-units", "general.units", "Int");
    this.setOption("ff-list-bars", "general.bar", "Char");
    var position = document.getElementById("ff-group-position");
    if (parseInt(position.selectedItem.value) == -1)
      this.mBranch.setIntPref("general.position", -1);
    else
      this.setOption("ff-text-position", "general.position", "Int");
  
    //swa panel
    this.setOption("ff-chk-swa-panel", "swa.panel.enabled", "Bool");
    this.setOption("ff-list-swa-panel-display", "swa.panel.display", "Int");
    
    //swa tooltip
    this.setOption("ff-chk-swa-tooltip", "swa.tooltip.enabled", "Bool");
    this.setOption("ff-list-swa-tooltip-display", "swa.tooltip.display", "Int");
    
    //swa slider
    this.setOption("ff-chk-swa-slider", "swa.slider.enabled", "Bool");
    this.setOption("ff-list-swa-slider-display", "swa.slider.display", "Int");
  
    //radar panel
    this.setOption("ff-chk-radar-panel", "radar.panel.enabled", "Bool");
    this.setOption("ff-list-radar-panel-display", "radar.panel.display", "Int");
    
    //radar tooltip
    this.setOption("ff-chk-radar-tooltip", "radar.tooltip.enabled", "Bool");
    this.setOption("ff-list-radar-tooltip-display", "radar.tooltip.display", "Int");
          
    //current conditions panel
    this.setOption("ff-chk-cc-panel", "cc.panel.enabled", "Bool");
    this.setOption("ff-list-cc-panel-display", "cc.panel.display", "Int");
    
    //current conditions tooltip
    this.setOption("ff-chk-cc-tooltip", "cc.tooltip.enabled", "Bool");
    this.setOption("ff-list-cc-tooltip-display", "cc.tooltip.display", "Int");
    
    //current conditions slider
    this.setOption("ff-chk-cc-slider", "cc.slider.enabled", "Bool");
    this.setOption("ff-list-cc-slider-display", "cc.slider.display", "Int");
    this.setOption("ff-text-cc-slider", "cc.slider.freq", "Int");
      
    //today's panel
    this.setOption("ff-chk-dayt-panel", "dayt.panel.enabled", "Bool");
    this.setOption("ff-list-dayt-panel-display", "dayt.panel.display", "Int");
    this.setOption("ff-list-dayt-panel-mode", "dayt.panel.mode", "Int");
      
    //today's tooltip
    this.setOption("ff-chk-dayt-tooltip", "dayt.tooltip.enabled", "Bool");
    this.setOption("ff-list-dayt-tooltip-display", "dayt.tooltip.display", "Int");
  
    //today's switch
    this.setOption("ff-list-dayt-switch", "dayt.panel.switch", "Int");
    
    //extended panel
    this.setOption("ff-text-dayf-days", "dayf.panel.days", "Int");
    this.setOption("ff-chk-dayf-panel", "dayf.panel.enabled", "Bool");
    this.setOption("ff-list-dayf-panel-display", "dayf.panel.display", "Int");
    this.setOption("ff-list-dayf-panel-mode", "dayf.panel.mode", "Int");
      
    //extended tooltip
    this.setOption("ff-chk-dayf-tooltip", "dayf.tooltip.enabled", "Bool");
    this.setOption("ff-list-dayf-tooltip-display", "dayf.tooltip.display", "Int");
    
    //profiles
    this.setOption("ff-chk-profs-rotate", "profile.switch.enabled", "Bool");
    this.setOption("ff-text-profs-rotate", "profile.switch.delay", "Int");
  },
  
  _encodeValue: function(aValue)
  {
    // only encode the strings
    if (typeof aValue != "string")
      return aValue;
    
    // prepare the quotes so that we can use eval
    return aValue.replace(/"/g,"\\\"").replace(/'/g,"\\\'");
  },
  
  importSettings: function()
  {
    this.mProfileLoading = true;
    this._manager.profiles.open(this._manager.profiles.OPEN_IMPORT);
    this.mProfileLoading = false;
    this.populateProfiles();
    this.populateOptions();
  },
  
  exportSettings: function()
  {
    this._manager.profiles.open(this._manager.profiles.OPEN_EXPORT);
  },
    
  observe: function(aSubject, aTopic, aData)
  {
    var method = null;
    if (aData.indexOf(",") != -1)
      method = aData.substring(0, aData.indexOf(","));
    else
      method = aData.substring(0, aData.length);
      
    if (method == "profile") {
      this.populateProfiles();
      this.populateOptions();
    }    
  },
    
  openLink: function(aEvent, aPartner)
  {
    var url = aEvent.currentTarget.getAttribute("url");
    if (aPartner)
      url = url + "?partner=" + this._manager.partner;
      
    //XXX need app specific way that doesn't use app.id pref
    if (this._guid != "{86c18b42-e466-45a9-ae7a-9b95ba6f5640}")
      window.openDialog("chrome://browser/content/browser.xul", "_blank", "chrome,all,dialog=no", url, null, null);
    else
      window.openDialog("chrome://navigator/content/navigator.xul", "_blank", "chrome,all,dialog=no", url, null, null);
  }
}
  
function updateButtons(aEvent)
{
  // don't enable the apply button for button presses
  if (aEvent) { 
    if ((aEvent.originalTarget.localName == "button") ||
        (aEvent.originalTarget.getAttribute("type") == "prof"))
    return;
  }
   
  document.getElementById("apply").removeAttribute("disabled");
}

function topWindow()
{
  //XXX need app specific  
  var windowManager = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService();
  var windowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator);
  var win = windowManagerInterface.getMostRecentWindow("navigator:browser");
  if (!win)
    win = window.openDialog("chrome://browser/content/browser.xul", "_blank", "chrome,all,dialog=no", "about:blank", null, null);
  return win;
}

function concat(c1, c2)
{
  // Concats too collections into an array.
  var c3 = new Array(c1.length + c2.length);
  var x,y = 0;

  for (x = 0; x < c1.length; x++)
    c3[y++] = c1[x];

  for (x = 0; x < c2.length; x++)
    c3[y++] = c2[x];

  return c3;
}
