/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is FoxyTunes Mozilla Extension and Engine.
 * The Initial Developer of the Original Code is Alex Sirota <alex@elbrus.com>. 
 * Portions created by Alex Sirota are Copyright (C) 2005 Alex Sirota. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function FoxyTunesTimeTools() {
	this.dateService = Components.classes["@mozilla.org/intl/scriptabledateformat;1"].
			getService(Components.interfaces.nsIScriptableDateFormat);
	this.alarmTimerId = -1;
	this.alarmTime = foxytunesReadSinglePreference('alarmclock_time', '');
	this.StartTimerIfNeeded();

	this.sleepRemainingTime = 0;
	this.sleepTimerId = -1;
}

FoxyTunesTimeTools.prototype = {
	CurrentFormattedTime : function() {
		var date = new Date();
		return this.dateService.FormatTime("", 
							this.dateService.timeFormatNoSeconds, 
							date.getHours(), 
							date.getMinutes(), 
							0);
    	},

	NormalizeTimeStr : function(time) {
		var hour;
		var min;
		var normalized;
		
		if (time.match(/^(\d+):([^ ]+)/)) {
			hour = RegExp.$1;
			if (hour.length == 1) {
				hour = "0" + hour;
			}
			min  = RegExp.$2;
			if (min.length == 1) {
				min = "0" + min;
			}
			normalized = hour + ":" + min;
			if (time.indexOf("AM") > -1) {
				normalized += "AM";
			}
			if (time.indexOf("PM") > -1) {
				normalized += "PM";
			}
		} else {
			dump("Warning: wrong time format: + " + time + "\n");
			return time;
		}

		return normalized;
	},

	DoCommand : function(cmd) {
		document.getElementById(cmd).doCommand();
	},

	AlarmClockTimerFunc : function() {
		var now = gFoxyTunesTimeTools.CurrentFormattedTime();
		//dump('in alarm proc ' + new Date() + '\n');
		if (gFoxyTunesTimeTools.NormalizeTimeStr(now) == 
		     gFoxyTunesTimeTools.NormalizeTimeStr(gFoxyTunesTimeTools.alarmTime)) {
			var bRecurring = foxytunesReadSinglePreference('alarmclock_recurring', 'false');
			if (bRecurring != 'true') {
				gFoxyTunesTimeTools.alarmTime = '';
				foxytunesWriteSinglePreference('alarmclock_time', '');
				clearInterval(gFoxyTunesTimeTools.alarmTimerId);
			} else {
				// recurring alarm, stop the timer for now
				// restart after a while to prevent triggering this event
				// right now instead of the same time tomorrow
				clearInterval(gFoxyTunesTimeTools.alarmTimerId);
				setTimeout('gFoxyTunesTimeTools.StartTimerIfNeeded()', 2000*60);
			}
			gFoxyTunesTimeTools.DoCommand('cmd_foxytunes-play');
		}
	},

	VerifyTimeString : function(time) {
		// get the current time for reference
		var now = this.CurrentFormattedTime();

		if ((now.indexOf("AM") > -1) || (now.indexOf("PM") > -1)) {
			// if there is a AM/PM in the reference time
			// verify that there is one in our time
			if ((time.indexOf("AM") == -1) && (time.indexOf("PM") == -1)) {
				return false;
			}
		}

		if (time.match(/^(\d+):([^ ]+)/)) {
			// check hours
			if (!((RegExp.$1 >= 0) && (RegExp.$1 <= 24))) {
				return false;
			}

			// check minutes
			if (!((RegExp.$2 >= 0) && (RegExp.$1 <= 59))) {
				return false;
			}
		} else {
			return false;
		}
		return true;
	},

	StartTimerIfNeeded : function() {
		if (this.alarmTime != '') {
			// check the time every 20 seconds
			this.alarmTimerId = setInterval(this.AlarmClockTimerFunc, 1000*20);
		}
	},

	UpdateAlarmTime : function(time) {
		this.alarmTime = time;
		foxytunesWriteSinglePreference('alarmclock_time', this.alarmTime);

		// stop the timer
		clearInterval(this.alarmTimerId);

		// restart the timer if needed
		this.StartTimerIfNeeded();

	},

	OpenAlarmDialog : function() {
		var now = this.CurrentFormattedTime();
		var status = '';
		if (this.alarmTime) {
			status = foxytunesGetLocaleString('alarmSetTo', [this.alarmTime]);
 		} else {
			status = foxytunesGetLocaleString('alarmNotSet');
		}
		window.openDialog('chrome://foxytunes/content/foxytunesAlarmClock.xul',
							'',
							'chrome,centerscreen,modal',
							this.alarmTime, now, status);
	},

	UpdateMenuItems : function () {
		var menuitem = document.getElementById('foxytunes-alarm-clock-menu-item');
		if (menuitem) {
			var bAlarmOn = this.alarmTime != '';
			menuitem.setAttribute('checked', bAlarmOn ? 'true' : 'false');
			if (bAlarmOn) {
				menuitem.setAttribute('label', menuitem.getAttribute('origlabel') + " - " + this.alarmTime);
			} else {
				menuitem.setAttribute('label', menuitem.getAttribute('origlabel'));
			}
		}

		var menuitem = document.getElementById('foxytunes-sleep-timer-menu-item');
		if (menuitem) {
			var bSleepOn = this.sleepRemainingTime > 0;
			menuitem.setAttribute('checked', bSleepOn ? 'true' : 'false');
			if (bSleepOn) {
				var displayTime = Math.max(1, Math.round(this.sleepRemainingTime / 60))
				menuitem.setAttribute('label', menuitem.getAttribute('origlabel') + " - " + 
						foxytunesGetLocaleString('xMinutesLeft', [displayTime]));
			} else {
				menuitem.setAttribute('label', menuitem.getAttribute('origlabel'));
			}
		}
	},

	// ***** Sleep code

	SleepTimerFunc : function() {
		//dump(gFoxyTunesTimeTools.sleepRemainingTime + '\n');
		gFoxyTunesTimeTools.sleepRemainingTime--;
		if (gFoxyTunesTimeTools.sleepRemainingTime <= 0) {
			clearInterval(gFoxyTunesTimeTools.sleepTimerId);
			gFoxyTunesTimeTools.sleepRemainingTime = 0;
			gFoxyTunesTimeTools.DoCommand('cmd_foxytunes-stop');
		}
	},

	UpdateSleepTimeout : function(timeout) {
		// the internal sleep timer is in seconds
		this.sleepRemainingTime = (timeout == '' ? 0 : timeout*60);

		// stop the timer
		clearInterval(this.sleepTimerId);

		// restart the timer if needed
		if (this.sleepRemainingTime > 0) {
			this.sleepTimerId = setInterval(this.SleepTimerFunc, 1000);
		}
	},

	OpenSleepDialog : function() {
		var time = '';
		if (this.sleepRemainingTime) {
			time = Math.max(1, Math.round(this.sleepRemainingTime / 60));
		}
		window.openDialog('chrome://foxytunes/content/foxytunesSleepTimer.xul',
							'',
							'chrome,centerscreen,modal',
							time);
	},
}

var gFoxyTunesTimeTools = new FoxyTunesTimeTools();
