// SuperScroll extension for Mozilla Firefox
// Copyright (C) 2004  Michael O'Rourke / Cosmic Cat Creations (http://www.cosmicat.com/)
// For licensing terms, please refer to readme.txt in this extension's XPInstall 
// package or its installation directory on your computer.

var SuperScroll = {
	init: function() {
		this.Browser = document.getElementById("content");
		this.Markers = new Array();
		this.Prefs = new SuperScroll_PrefManager();
		this.setSmoothPref();
		this.setScrollDistance();
		this.setScrollInterval();
		this.setPageTurn();
		this.setKeyOverrides();
		this.setTypeAheadFind();
		this.setListener();
		this.Prefs.addPrefObserver("SuperScroll.prefObserver", "root");
	},
	destruct: function() {
		try { this.Prefs.removePrefObserver(); } catch(ex) {}
	},
	getPref: function(strName) {
		return this.Prefs.getPref(strName);
	},
	prefObserver: function(subject, topic, prefName) {
		switch (prefName) {
			case "general.smoothScroll": this.setSmoothPref(); break;
			case "superscroll.keyscroll": this.setListener(); break;
			case "superscroll.numlines": this.setScrollDistance(); break;
			case "superscroll.scrollspeed": this.setScrollInterval(); break;
			case "superscroll.keyoverrides": this.setKeyOverrides(); break;
			case "superscroll.pageturn": this.setPageTurn(); break;
			case "accessibility.typeaheadfind": this.setTypeAheadFind(); break;
		}
	},
	setSmoothPref: function() {
		this.smoothPref = this.getPref("general.smoothScroll");
	},
	setScrollDistance: function() {
		var numLines = Math.max(1, this.getPref("superscroll.numlines"));
		this.scrollDistance = Math.round(this.getLineHeight() * numLines);
	},
	setScrollInterval: function() {
		this.scrollInterval = this.getPref("superscroll.scrollspeed");
	},
	setPageTurn: function() {
		this.pageTurn = Number(this.getPref("superscroll.pageturn")) / 100;
	},
	setKeyOverrides: function() {
		var arrKeys = String(this.getPref("superscroll.keyoverrides")).split(",");
		this.KeyUp = this.getOverrideKey(0, arrKeys);
		this.KeyRight = this.getOverrideKey(1, arrKeys);
		this.KeyDown = this.getOverrideKey(2, arrKeys);
		this.KeyLeft = this.getOverrideKey(3, arrKeys);
		this.KeyPageUp = this.getOverrideKey(4, arrKeys);
		this.KeyPageDown = this.getOverrideKey(5, arrKeys);
		this.KeyHome = this.getOverrideKey(6, arrKeys);
		this.KeyEnd = this.getOverrideKey(7, arrKeys);
		if (this.KeyUp[0] || this.KeyRight[0] || this.KeyDown[0] || this.KeyLeft[0]
				|| this.KeyPageUp[0] || this.KeyPageDown[0] || this.KeyHome[0] || this.KeyEnd[0]) {
			this.Browser.addEventListener("keypress", SuperScroll.doKeyMaster, true);
		} else {
			this.Browser.removeEventListener("keypress", SuperScroll.doKeyMaster, true);
		}
	},
	setTypeAheadFind: function() {
		this.typeAheadFind = this.getPref("accessibility.typeaheadfind");
	},
	setListener: function() {
		if (this.getPref("superscroll.keyscroll", false)) {
			this.Browser.addEventListener("keypress", SuperScroll.doKeyScroller, false);
		} else {
			this.Browser.removeEventListener("keypress", SuperScroll.doKeyScroller, false);
		}
	},
	doKeyMaster: function(event) {
		SuperScroll.KeyMaster(event);
	},
	KeyMaster: function(event) {		
		if (this.typeAheadFind && this.isTypeAheadFindActive()) { return; }
		var code = (event.keyCode) ? event.keyCode : event.charCode;
		switch (true) {
			case ((code == this.KeyDown[0]) && (this.checkKeyMods(event, this.KeyDown[1]))):
				this.doKeyEventOverride(event, new Array(KeyEvent.DOM_VK_DOWN, 0, false, false, false)); break;
			case ((code == this.KeyUp[0]) && (this.checkKeyMods(event, this.KeyUp[1]))):
				this.doKeyEventOverride(event, new Array(KeyEvent.DOM_VK_UP, 0, false, false, false)); break;
			case ((code == this.KeyRight[0]) && (this.checkKeyMods(event, this.KeyRight[1]))):
				this.doKeyEventOverride(event, new Array(KeyEvent.DOM_VK_RIGHT, 0, false, false, false)); break;
			case ((code == this.KeyLeft[0]) && (this.checkKeyMods(event, this.KeyLeft[1]))):
				this.doKeyEventOverride(event, new Array(KeyEvent.DOM_VK_LEFT, 0, false, false, false)); break;
			case ((code == this.KeyPageDown[0]) && (this.checkKeyMods(event, this.KeyPageDown[1]))):
				this.doKeyEventOverride(event, new Array(KeyEvent.DOM_VK_PAGE_DOWN, 0, false, false, false)); break;
			case ((code == this.KeyPageUp[0]) && (this.checkKeyMods(event, this.KeyPageUp[1]))):
				this.doKeyEventOverride(event, new Array(KeyEvent.DOM_VK_PAGE_UP, 0, false, false, false)); break;
			case ((code == this.KeyEnd[0]) && (this.checkKeyMods(event, this.KeyEnd[1]))):
				this.doKeyEventOverride(event, new Array(KeyEvent.DOM_VK_END, 0, false, false, false)); break;
			case ((code == this.KeyHome[0]) && (this.checkKeyMods(event, this.KeyHome[1]))):
				this.doKeyEventOverride(event, new Array(KeyEvent.DOM_VK_HOME, 0, false, false, false)); break;
		}
	},
	doKeyScroller: function(event) {
		SuperScroll.KeyScroller(event);
	},
	KeyScroller: function(event) {
		if (event.altKey) { return; }
		switch (event.keyCode) {
			case KeyEvent.DOM_VK_DOWN: this.scrollView("y", event, 1); break;
			case KeyEvent.DOM_VK_UP: this.scrollView("y", event, -1); break;
			case KeyEvent.DOM_VK_RIGHT: this.scrollView("x", event, 1); break;
			case KeyEvent.DOM_VK_LEFT: this.scrollView("x", event, -1); break;
			case KeyEvent.DOM_VK_PAGE_DOWN: this.scrollPageUpDown(event, 1); break;
			case KeyEvent.DOM_VK_PAGE_UP: this.scrollPageUpDown(event, -1); break;
			case KeyEvent.DOM_VK_END: this.scrollPageTopBottom(event, "cmd_scrollBottom"); break;
			case KeyEvent.DOM_VK_HOME: this.scrollPageTopBottom(event, "cmd_scrollTop"); break;
			default:
				if (event.charCode == KeyEvent.DOM_VK_SPACE) {
					if (!this.isViewScrollable(event.originalTarget)) { return; }
					if (!event.shiftKey) {
						this.scrollPageUpDown(event, 1);
					} else {
						this.scrollPageUpDown(event, -1);
					}
				}
		}
	},
	scrollView: function(axis, event, direction) {
		if (!this.isViewScrollable(event.originalTarget)) { return; }
		event.stopPropagation();
		event.preventDefault();
		var oView = event.explicitOriginalTarget.ownerDocument.defaultView;
		var x = (this.smoothPref) ? this.scrollInterval * direction : this.scrollDistance * direction;
		var i, xAbs = Math.abs(x);
		if (axis == "y") {
			for (i=0; i < this.scrollDistance; i += xAbs) { oView.scrollBy(0, x); }
		} else {
			for (i=0; i < this.scrollDistance; i += xAbs) { oView.scrollBy(x, 0); }
		}
	},
	scrollPageUpDown: function(event, direction) {
		if (!this.isViewScrollable(event.originalTarget)) { return; }
		event.stopPropagation();
		event.preventDefault();
		var oView = event.explicitOriginalTarget.ownerDocument.defaultView;
		var scrollDistance = Math.round(oView.innerHeight * this.pageTurn);
		var x = (this.smoothPref) ? (Math.floor(scrollDistance / 16) * direction) : (scrollDistance * direction);
		var xAbs = Math.abs(x);
		if (this.smoothPref) {
			for (var i=0; i < 16; i++) { oView.scrollBy(0, x); }
			oView.scrollBy(0, (scrollDistance * direction) - (x * 16));
		} else {
			oView.scrollBy(0, x);
		}
	},
	scrollPageTopBottom: function(event, command) {
		if (!this.isViewScrollable(event.originalTarget)) { return; }
		event.stopPropagation();
		event.preventDefault();
		goDoCommand(command);
	},
	isViewScrollable: function(hNode) {
		if (!hNode) { return true; }
		switch (hNode.nodeName) {
			case "INPUT": return ((hNode.type == "text") || (hNode.type == "password")) ? false : true; break;
			case "TEXTAREA": case "SELECT": case "AREA": case "html:input": return false; break;
		}
		return true;
	},
	isTypeAheadFindActive: function() {
		try {
			var typeAhead = Components.classes["@mozilla.org/typeaheadfind;1"].getService(Components.interfaces.nsITypeAheadFind);
			return typeAhead.isActive;
		} catch(ex) { return false; }
	},
	getLineHeight: function() {
		/* the following line returns a chrome URI instead of the pref's string value:
		var prefFontGroup = this.getPref("font.language.group"); */
		var prefFontGroup = "x-western";
		var prefFontSize = this.getPref("font.size.variable." + prefFontGroup);
		return prefFontSize * 1.1;
	},
	checkKeyMods: function(event, keyMods) {
		switch (false) {
			case (Boolean(keyMods & 1) == Boolean(event.charCode)):
			case (Boolean(keyMods & 2) == event.altKey):
			case (Boolean(keyMods & 4) == event.ctrlKey):
			case (Boolean(keyMods & 8) == event.shiftKey):
				return false;
		}
		return true;
	},
	makeArrNumeric: function(arrIn) {
		for (var i=0; i < arrIn.length; i++) { arrIn[i] = Number(arrIn[i]); }
		return arrIn;
	},
	getOverrideKey: function(i, arrKeys) {
		return (arrKeys.length > i) ? this.makeArrNumeric(arrKeys[i].split(":")) : new Array(0,0);
	},
	doKeyEventOverride: function(event, arrKey) {
		if (!this.isViewScrollable(event.originalTarget)) { return; }
		event.stopPropagation();
		event.preventDefault();
		var oDoc = event.explicitOriginalTarget.ownerDocument;
		var e = oDoc.createEvent("KeyEvents");
		e.initKeyEvent("keypress", true, true, oDoc.defaultView, arrKey[2], arrKey[3], arrKey[4], false, arrKey[0], arrKey[1]);
		e.superscrollCustom = true;
		oDoc.dispatchEvent(e);
	}
}

window.addEventListener("load", function() { SuperScroll.init(); }, false);
window.addEventListener("unload", function() { SuperScroll.destruct(); }, false);
