var gPS = new SuperScroll_PrefManager();
var dialog;

function initDialogObject() {
	moveToAlertPosition();
	dialog = new Object;
	dialog.keyPref = document.getElementById("dialog.keyPref");
	dialog.keyNumLines = document.getElementById("dialog.keyNumLines");
	dialog.wheelPref = document.getElementById("dialog.wheelPref");
	dialog.wheelNumLines = document.getElementById("dialog.wheelNumLines");
	
	dialog.vk_up = document.getElementById("dialog.vk_up");
	dialog.vk_left = document.getElementById("dialog.vk_left");
	dialog.vk_right = document.getElementById("dialog.vk_right");
	dialog.vk_down = document.getElementById("dialog.vk_down");
	dialog.vk_pageup = document.getElementById("dialog.vk_pageup");
	dialog.vk_pagedown = document.getElementById("dialog.vk_pagedown");
	dialog.vk_home = document.getElementById("dialog.vk_home");
	dialog.vk_end = document.getElementById("dialog.vk_end");
	
	dialog.keyNumLines.value = gPS.getPref("superscroll.numlines");
	dialog.wheelNumLines.value = gPS.getPref("mousewheel.withnokey.numlines");
	
	try {
		var keyPref = gPS.getPref("superscroll.keyscroll");
		keyPref = (keyPref) ? "keyPrefTrue" : "keyPrefFalse";
		dialog.keyPref.selectedItem = document.getElementById(keyPref);
		
		var wheelPref = gPS.getPref("mousewheel.withnokey.sysnumlines");
		wheelPref = (wheelPref) ? "wheelPrefTrue" : "wheelPrefFalse";
		dialog.wheelPref.selectedItem = document.getElementById(wheelPref);
	} catch(err) { alert(err); }
	
	var arrKeys = gPS.getPref("superscroll.keyoverrides").split(",");
	dialog.overrideUp = getOverrideKey(0, arrKeys);
	dialog.overrideRight = getOverrideKey(1, arrKeys);
	dialog.overrideDown = getOverrideKey(2, arrKeys);
	dialog.overrideLeft = getOverrideKey(3, arrKeys);
	dialog.overridePageUp = getOverrideKey(4, arrKeys);
	dialog.overridePageDown = getOverrideKey(5, arrKeys);
	dialog.overrideHome = getOverrideKey(6, arrKeys);
	dialog.overrideEnd = getOverrideKey(7, arrKeys);
	dialog.vk_up.value = makeKeyString(dialog.overrideUp);
	dialog.vk_right.value = makeKeyString(dialog.overrideRight);
	dialog.vk_down.value = makeKeyString(dialog.overrideDown);
	dialog.vk_left.value = makeKeyString(dialog.overrideLeft);
	dialog.vk_pageup.value = makeKeyString(dialog.overridePageUp);
	dialog.vk_pagedown.value = makeKeyString(dialog.overridePageDown);
	dialog.vk_home.value = makeKeyString(dialog.overrideHome);
	dialog.vk_end.value = makeKeyString(dialog.overrideEnd);
	dialog.vk_up.setAttribute("prefstring", dialog.overrideUp.join(":"));
	dialog.vk_right.setAttribute("prefstring", dialog.overrideRight.join(":"));
	dialog.vk_down.setAttribute("prefstring", dialog.overrideDown.join(":"));
	dialog.vk_left.setAttribute("prefstring", dialog.overrideLeft.join(":"));
	dialog.vk_pageup.setAttribute("prefstring", dialog.overridePageUp.join(":"));
	dialog.vk_pagedown.setAttribute("prefstring", dialog.overridePageDown.join(":"));
	dialog.vk_home.setAttribute("prefstring", dialog.overrideHome.join(":"));
	dialog.vk_end.setAttribute("prefstring", dialog.overrideEnd.join(":"));
}

function onLoad() {
	initDialogObject();
}

function onAccept() {
	var numLines;
	if (!isNaN(dialog.keyNumLines.value)) {
		numLines = dialog.keyNumLines.value;
		if (numLines < 1) { numLines = 1; } else if (numLines > 100) { numLines = 100; }
		gPS.setPref("superscroll.numlines", numLines);
	}
	if (!isNaN(dialog.wheelNumLines.value)) {
		numLines = dialog.wheelNumLines.value;
		if (numLines < 1) { numLines = 1; } else if (numLines > 100) { numLines = 100; }
		gPS.setPref("mousewheel.withnokey.numlines", numLines);
	}
	try {
		var keyPref = (dialog.keyPref.selectedItem.value == "true") ? true : false;
		gPS.setPref("superscroll.keyscroll", keyPref);
		var wheelPref = (dialog.wheelPref.selectedItem.value == "true") ? true : false;
		gPS.setPref("mousewheel.withnokey.sysnumlines", wheelPref);
	} catch(err) { alert(err); }
	
	var keyPrefString = dialog.vk_up.getAttribute("prefstring") + ","
				+ dialog.vk_right.getAttribute("prefstring") + ","
				+ dialog.vk_down.getAttribute("prefstring") + ","
				+ dialog.vk_left.getAttribute("prefstring") + ","
				+ dialog.vk_pageup.getAttribute("prefstring") + ","
				+ dialog.vk_pagedown.getAttribute("prefstring") + ","
				+ dialog.vk_home.getAttribute("prefstring") + ","
				+ dialog.vk_end.getAttribute("prefstring");
	gPS.setPref("superscroll.keyoverrides", keyPrefString);
	return true;
}

function grabKey(event) {
	switch (event.keyCode) {
		//ignore these keys, so they don't interfere with normal dialog functions
		case KeyEvent.DOM_VK_TAB:
		case KeyEvent.DOM_VK_ENTER:
		case KeyEvent.DOM_VK_RETURN:
		case KeyEvent.DOM_VK_ESCAPE:
			return;
	}
	var sTemp = "", sPref = "", keyMods = 0;
	if (event.altKey) {
		keyMods += 2;
		sTemp += "ALT + ";
	}
	if (event.ctrlKey) {
		keyMods += 4;
		sTemp += "CTRL + ";
	}
	if (event.shiftKey) {
		keyMods += 8;
		if (event.charCode == 0) { sTemp += "SHIFT + "; }
	}
	if (event.charCode != 0) {
		keyMods += 1;
		sTemp += (event.charCode != KeyEvent.DOM_VK_SPACE) ? String.fromCharCode(event.charCode) : "SPACE";
//		sTemp += String.fromCharCode(event.charCode);
		sPref = event.charCode + ":" + keyMods;
	} else {
		sTemp += getKeyCodeEntity(event.keyCode);
		sPref = event.keyCode + ":" + keyMods;
	}
	event.target.value = sTemp;
	event.target.setAttribute("prefstring", sPref);
	event.preventDefault();
}

function makeKeyString(arrKey) {
	if (arrKey.length < 2) { arrKey[0] = ""; }
	if (arrKey[0] == "") { return ""; }
	var sTemp = "";
	if (arrKey[1] & 2) { sTemp += "ALT + "; }
	if (arrKey[1] & 4) { sTemp += "CTRL + "; }
	if ((arrKey[1] & 8) && !(arrKey[1] & 1)) { sTemp += "SHIFT + "; }
	if (arrKey[1] & 1) {
		sTemp += (arrKey[0] != KeyEvent.DOM_VK_SPACE) ? String.fromCharCode(arrKey[0]) : "SPACE";
	} else {
		sTemp += getKeyCodeEntity(arrKey[0]);
	}
	return sTemp;
}

function makeNumeric(arrKey) {
	for (var i=0; i < arrKey.length; i++) { arrKey[i] = Number(arrKey[i]); }
	return arrKey;
}

function getOverrideKey(i, arrKeys) {
	return (arrKeys.length > i) ? makeNumeric(arrKeys[i].split(":")) : new Array("","");
}

function getKeyCodeEntity(keyCode) {
	switch (keyCode) {
		case KeyEvent.DOM_VK_CANCEL: return "CANCEL"; break;
		case KeyEvent.DOM_VK_BACK_SPACE: return "BACKSPACE"; break;
		case KeyEvent.DOM_VK_TAB: return "TAB"; break;
		case KeyEvent.DOM_VK_CLEAR: return "CLEAR"; break;
		case KeyEvent.DOM_VK_ENTER: return "ENTER"; break;
		case KeyEvent.DOM_VK_RETURN: return "RETURN"; break;
		case KeyEvent.DOM_VK_PAUSE: return "PAUSE"; break;
		case KeyEvent.DOM_VK_SPACE: return "SPACE"; break;
		case KeyEvent.DOM_VK_PAGE_UP: return "PAGE UP"; break;
		case KeyEvent.DOM_VK_PAGE_DOWN: return "PAGE DOWN"; break;
		case KeyEvent.DOM_VK_END: return "END"; break;
		case KeyEvent.DOM_VK_HOME: return "HOME"; break;
		case KeyEvent.DOM_VK_PRINTSCREEN: return "PRINT SCREEN"; break;
		case KeyEvent.DOM_VK_INSERT: return "INSERT"; break;
		case KeyEvent.DOM_VK_DELETE: return "DELETE"; break;
		case KeyEvent.DOM_VK_DOWN: return "DOWN"; break;
		case KeyEvent.DOM_VK_UP: return "UP"; break;
		case KeyEvent.DOM_VK_LEFT: return "LEFT"; break;
		case KeyEvent.DOM_VK_RIGHT: return "RIGHT"; break;
		case KeyEvent.DOM_VK_F1: return "F1"; break;
		case KeyEvent.DOM_VK_F2: return "F2"; break;
		case KeyEvent.DOM_VK_F3: return "F3"; break;
		case KeyEvent.DOM_VK_F4: return "F4"; break;
		case KeyEvent.DOM_VK_F5: return "F5"; break;
		case KeyEvent.DOM_VK_F6: return "F6"; break;
		case KeyEvent.DOM_VK_F7: return "F7"; break;
		case KeyEvent.DOM_VK_F8: return "F8"; break;
		case KeyEvent.DOM_VK_F9: return "F9"; break;
		case KeyEvent.DOM_VK_F10: return "F10"; break;
		case KeyEvent.DOM_VK_F11: return "F11"; break;
		case KeyEvent.DOM_VK_F12: return "F12"; break;
		case KeyEvent.DOM_VK_F13: return "F13"; break;
		case KeyEvent.DOM_VK_F14: return "F14"; break;
		case KeyEvent.DOM_VK_F15: return "F15"; break;
		case KeyEvent.DOM_VK_F16: return "F16"; break;
		case KeyEvent.DOM_VK_F17: return "F17"; break;
		case KeyEvent.DOM_VK_F18: return "F18"; break;
		case KeyEvent.DOM_VK_F19: return "F19"; break;
		case KeyEvent.DOM_VK_F20: return "F20"; break;
		case KeyEvent.DOM_VK_F21: return "F21"; break;
		case KeyEvent.DOM_VK_F22: return "F22"; break;
		case KeyEvent.DOM_VK_F23: return "F23"; break;
		case KeyEvent.DOM_VK_F24: return "F24"; break;
		case KeyEvent.DOM_VK_HELP: return "HELP"; break;
	}
	return "?";
}

function clearKey(strElement) {
	var hElement = document.getElementById(strElement);
	hElement.value = "";
	hElement.setAttribute("prefstring", "");
}
