/*
 * tabmix.js
 *
 * original code by Bradley Chapman
 * modified and developped by Hemiola SUN
 */
var gPref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
var gObserver = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
var WindowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);

var __contentAreaClick = window.contentAreaClick;
var __Startup = (typeof(window.BrowserStartup)!='undefined') ? window.BrowserStartup : window.Startup; 
var __BrowserOpenTab = window.BrowserOpenTab;
var __BrowserLoadURL = window.BrowserLoadURL;
var __SearchLoadURL = window.SearchLoadURL;

/*
 * setup the extension core
 *
 * PRECONDITION: None.
 * POSTCONDITION: None.
 *
 */
function TBPL_init()
{
  var hBrowser = document.getElementById("help-content");
  if (hBrowser) {
      hBrowser.addEventListener("click", TBP_helpContentClick, true);
  }

  window.contentAreaClick = TBPL_contentAreaClick;
  window.BrowserOpenTab = TBP_BrowserOpenTab;
  window.BrowserLoadURL = TBP_BrowserLoadURL;
  window.SearchLoadURL = TBP_SearchLoadURL;
}

/*
 * tear down the observers inside the extension core
 *
 * PRECONDITION: None.
 * POSTCONDITION: Nine.
 *
 */
function TBPL_exit()
{
    var hBrowser = document.getElementById("help-content");

    if (hBrowser) {
        hBrowser.removeEventListener("click", TBP_helpContentClick, true);
    }

    window.BrowserOpenTab = __BrowserOpenTab;
    window.contentAreaClick = __contentAreaClick;
    window.BrowserLoadURL = __BrowserLoadURL;
    window.SearchLoadURL = __SearchLoadURL;

    return;
}

/*
 * force-call the window observer at least one time
 *
 * PRECONDITION: None.
 * POSTCONDITION: None.
 *
 */
function TBPL_Startup()
{
    // force-call the observer once, in order to kill new windows faster
    TBPL_DOMWindowOpenObserver.onObserve(window, TBPL_DOMWindowOpenObserver);

  __Startup();

    return;
}

function TM_howToOpen (event, linkNode, currentTab)
{
  if ( event.altKey || event.shiftKey )
    return handleLinkClick(event, linkNode.href, linkNode);
  if ( event.ctrlKey || event.metaKey )
    currentTab = !currentTab;

  if ( currentTab )
    window.loadURI(linkNode.href);
  else
    window.openNewTabWith(linkNode.href, linkNode, event, true);

  event.preventDefault();
  event.stopPropagation();
  return true;
}

// this handles special-case clicks
/*
 * sort and triage the special-cased left-clicks inside the content area of a tabbrowser
 *
 * PRECONDITION: event must be a valid reference to an event union from the
 *   central dispatch framework. fieldNormalClicks must be a Boolean value of
 *   either true or false.
 * POSTCONDITION: If certain conditions are met the function returns the return
 *   value of __contentAreaClick, the original Firefox function.
 *
 */
function TBPL_contentAreaClick(event, fieldNormalClicks)
{
  try {
    var targetPref = gPref.getIntPref("extensions.tabmix.speLink");
    var linkTarget = gPref.getBoolPref("extensions.tabmix.linkTarget");
  }
  catch(e) {}
  // use the default if either of these are activated
  if ( targetPref == 0 && !gBrowser.mCurrentTab.hasAttribute("locked") && !linkTarget )
    return __contentAreaClick(event, fieldNormalClicks);

  var target = event.target;
  var linkNode;
  var local_name = target.localName;
  if (local_name)
    local_name = local_name.toLowerCase();

  switch (local_name) {
    case "a":
    case "area":
    case "link":
      if (target.hasAttribute("href"))
        linkNode = target;
      break;
    default:
      linkNode = findParentNode(event.originalTarget, "a");
      if (linkNode && !linkNode.hasAttribute("href"))
        linkNode = null;
      else if (linkNode && linkNode.hasAttribute("href"))
        target = linkNode;
      break;
  }
  if (!linkNode)
    return __contentAreaClick(event, fieldNormalClicks);
    



  // Check if link refers to external domain.
  // Get current page url
  var curpage = getWebNavigation().currentURI.spec;
  // Check if link contains http
  var linkregex = /^http/i;
  var match = linkregex.exec(target);
  if (match != null)
  {
    var re = new RegExp('([^.]*\.([^/]*))');
    var u = re.exec(target);
    var c = re.exec(curpage);
    u = u[2];
    c = c[2];
  }

  var openT = linkNode.getAttribute("target");
  // If link has no target attribute, check if there is a <base> with a target attribute
  if (!openT) {
    var b = document.commandDispatcher.focusedWindow.document.getElementsByTagName("base");
    if (b.length > 0)
      openT = b[0].getAttribute("target");
  }


  if ( event.button == 0 ) {
    if ( linkTarget && openT &&
         openT != "_self" && openT != "_parent" && openT != "_top" &&
         openT != "_content" && openT != "_main" &&
         !TBPL_existsFrameName(document.commandDispatcher.focusedWindow.top.frames, openT) ) {
        TM_howToOpen(event, linkNode, true);
    }

    else if ( targetPref == 2 && !openT && u && u.toString() != c.toString() ) { // by Stuart Colville
      TM_howToOpen(event, linkNode, false);
    }

    else if ( gBrowser.mCurrentTab.hasAttribute("locked") ) { // tab is locked
      if ( ( target.hasAttribute("href") && target.getAttribute("href").indexOf("javascript:") == 0 ) ||
           ( target.hasAttribute("onclick") && target.getAttribute("onclick").indexOf("window.open") == 0 ) )
        ; // javascript links, do nothing!
      else {
        var current = target.hasAttribute("href") && target.getAttribute("href").indexOf("#") == 0;
        TM_howToOpen(event, linkNode, current);
      }
    }

    else if ( !event.ctrlKey && !event.shiftKey && !event.altKey && !event.metaKey) {
      if (fieldNormalClicks && (!openT || openT == "_content" || openT  == "_main"))
        __contentAreaClick(event, fieldNormalClicks);
      else if (linkNode.getAttribute("rel") == "sidebar")
        __contentAreaClick(event, fieldNormalClicks);
      else if (openT == "_search")
        __contentAreaClick(event, fieldNormalClicks);
      else if (linkNode.getAttribute("onclick"))
        __contentAreaClick(event, fieldNormalClicks);

      //disable tab mix completely for mailto: links

      else if (linkNode.getAttribute("href").indexOf("mailto:") > -1)
        __contentAreaClick(event, fieldNormalClicks);
      else if (openT)
        handleLinkClick(event, linkNode.href, linkNode);
    }
    else
      handleLinkClick(event, linkNode.href, linkNode);

  } else { // event.button != 0
    handleLinkClick(event, linkNode.href, linkNode);
  }

  return true;
}

/*
 * check a document's frame pool and determine if targetFrame is located inside of it
 *
 * PRECONDITION: containerFrame must be a valid reference to the current frame pool of the
 *  current document. targetFrame must be the frame that we are seeking.
 * POSTCONDITION: None.
 *
 */
function TBPL_existsFrameName(containerFrame, targetFrame)
{
    for (var i = 0; i < containerFrame.length; ++i) {
          if (containerFrame[i].name == targetFrame) return true;
          if (containerFrame[i].frames.length) var return_var = TBPL_existsFrameName(containerFrame[i].frames,targetFrame);
    }

    if (return_var) return return_var;
    return false;
}

/*
 * redefine chrome://mozapps/content/extensions/extensions.js:openURL()
 * and handle links called from the EM and TM appropriately
 *
 * PRECONDITION: aURL must be a valid URI.
 * POSTCONDITION: None.
 *
 */
function TBPL_openURL(aURL)
{
    try {
          var targetPref = gPref.getIntPref("browser.link.open_newwindow");
    }
    catch (e) {}

    if ( aURL == null )
      aURL = "about:blank";

    // check for an existing window and focus it; it's not application modal
    var anyWindow = WindowManager.getMostRecentWindow("navigator:browser");

    if (anyWindow) {
        // get the charset of the current document, if it exists
        var originCharset;
        if (anyWindow.getBrowser() && anyWindow.getBrowser().characterSet) {
            originCharset = anyWindow.getBrowser().characterSet;
        }

        switch (targetPref) {
            case 2 : {
              anyWindow.openNewWindowWith(aURL, null, true, null);
                break;
            }
            case 3 : {
              var newTab = anyWindow.getBrowser().addTab(aURL, null, originCharset);
              anyWindow.getBrowser().selectedTab = newTab;
              break;
            }
            case 1 : {
              anyWindow.loadURI(aURL, null, originCharset);
              break;
            }
        }
    }
    else {
          // open browser.xul anyway
    anyWindow.openNewWindowWith(aURL, null, true, null);
    }
    return;
}


/*
 * iterate the list of "navigator:browser"s and return a window different
 * from aExclude (and not a popup window)
 *
 * PRECONDITION: aExclude must be a valid reference to a JavaScript representation of
 *  a browser window that must not be used by the caller.
 * POSTCONDITION: Returns a window that is not a popup window and is not aExclude.
 *
 */
function TBPL_getBrowserWindow(aExclude)
{
    var windows = WindowManager.getEnumerator('navigator:browser');

    while (windows.hasMoreElements()) {
        var win = windows.getNext().QueryInterface(Components.interfaces.nsIDOMWindow);
        if( (win != aExclude) )
           return win;
    }
    return null;
}

/*
 * handle all DOM window open events and catch attempts to open new windows
 *
 * PRECONDITION: None.
 * POSTCONDITION: None.
 *
 */
var TBPL_DOMWindowOpenObserver = {
    Tbplite: 'tbpl',

    observe : function(aSubject, aTopic, aData)
    {
        if (aTopic != 'domwindowopened') return;
        this.onObserve(aSubject, this);
        return;
    },

    onObserve : function(aSubject, aThis)
    {
        // get the preferences
        try {
          var targetPref = gPref.getBoolPref("extensions.tabmix.singleWindow");
          var linkOpen = gPref.getIntPref("browser.link.open_newwindow");
          var jsLinkOpen = gPref.getIntPref("browser.link.open_newwindow.restriction");
          var externalLinkOpen = gPref.getIntPref("browser.link.open_external");
        }
        catch(e){}

        if ( linkOpen == 2 || jsLinkOpen != 0 || externalLinkOpen == 2 ) {
          gPref.setBoolPref("extensions.tabmix.singleWindow", false);
          return;
        }
        if ( !targetPref )
          return;

        var newWindow = aSubject;
        var existingWindow = TBPL_getBrowserWindow(newWindow);

        // no navigator:browser window open yet?
        if (!existingWindow) return;

        // if the href is missing, try again later (xxx)
        if (!newWindow.location.href) {
          existingWindow.setTimeout(aThis.onObserve, 0, newWindow, aThis);
            return;
        }

        // we don't want to open non-browser windows in a tab
        if(newWindow.location.href != "chrome://browser/content/browser.xul")
            return;

        if ( !('arguments' in newWindow) || newWindow.arguments.length == 0 )
          return;

        var newTab = existingWindow.getBrowser().addTab(newWindow.arguments[0], null, null);
        existingWindow.getBrowser().selectedTab = newTab;
        newWindow.close();
        return;
    }
}
// end of TBPL_DOMWindowOpenObserver

// code from Single Window 1.4 by Aaron
function TBP_BrowserOpenTab()
{
    try {
      var targetPref = (gPref.getBoolPref("extensions.tabmix.loadHome"));
    }
    catch(e) {}

    if ( targetPref == true) {
      gBrowser.selectedTab = gBrowser.addTab(gHomeButton.getHomePage().split("|")[0]);
    }
    else {
      __BrowserOpenTab();
    }
  return;
}

// open a link in the help window, original code of TBP 1.0.0
function TBP_helpContentClick(event)
{
      if (event.button == 0 && !event.shiftKey && !event.ctrlKey && event.originalTarget.nodeName.toLowerCase() == "a") {
          var href = event.originalTarget.getAttribute("href");
          if (href.substr(0,5) == "http:" || href.substr(0,4) == "ftp:") {
                TBPL_openURL(href);
                event.preventDefault();
                return true;
          }
      }
    return false;
}

// open a new tab from the searchbar
function TBP_SearchLoadURL(URI, aTriggeringEvent)
{
  var theURI = URI;
  var theEvent = aTriggeringEvent;
  var thePostData, theTabPref, theBar, searchBar;
  var elementID = "searchbar";

  try {
          theTabPref = gPref.getBoolPref("extensions.tabmix.opentabfor.search");
  }
  catch(e) {}

  searchBar = document.getElementById(elementID);
  theBar = searchBar.getElementsByAttribute("class", "searchbar-textbox")[0];
  // if the current tab is empty, then do not open a new tab
  if (gBrowser.selectedBrowser.webNavigation.currentURI.spec == "about:blank") {
    theTabPref = false;
  }
  __TBP_LoadBarURL(elementID, theBar, theURI, theEvent, theTabPref, gPref.getBoolPref("browser.tabs.loadUrlInBackground"), thePostData);
  return;
}

// open a new tab from the urlbar
function TBP_BrowserLoadURL(aTriggeringEvent, aPostData)
{
  var theEvent = aTriggeringEvent;
  var thePostData = aPostData;
  var theURI, theTabPref, theBar;

  try {
          theTabPref = gPref.getBoolPref("browser.tabs.opentabfor.urlbar");
  }
  catch(e) {}

  // this hack is needed to make things work on older builds
  // it walks the parentNode listing until it reachs the toolbaritem for
  // either the URL bar or the searchbar, and then saves the element ID
  if (theEvent && theEvent.originalTarget) {
    var node = theEvent.originalTarget;
    var elementID;
    while (node && node.localname != "toolbaritem") node = node.parentNode;
    if (node) {
      elementID = node.childNodes[0].id;
    }
    else {
      elementID = "urlbar";
    }
  }
  else {
    // theEvent is missing - this means that somebody tried to open a URL
    // by clicking on the history dropdown
    elementID = "urlbar";
  }

  // if the current tab is empty, then do not open a new tab
  if (gBrowser.selectedBrowser.webNavigation.currentURI.spec == "about:blank") {
    theTabPref = false;
  }

  theBar = document.getElementById(elementID);
  theURI = theBar.value;
  __TBP_LoadBarURL(elementID, theBar, theURI, theEvent, theTabPref, gPref.getBoolPref("browser.tabs.loadUrlInBackground"), thePostData);

  return;
}

// code for searchbar and urlbar
function __TBP_LoadBarURL(elementID, theBar, theURI, theEvent, theTabPref, BackgroundPref, thePostData)
{
    // get the charset of the current document
    var wintype = document.firstChild.getAttribute("windowtype");
    var originCharset;
    if (wintype == "navigator:browser") {
          originCharset = window._content.document.characterSet;
    }

  // if we are viewing source, then do so
    if (theURI.match(/^view-source:/)) {
        BrowserViewSourceOfURL(theURI.replace(/^view-source:/, ""), null, null);
    return;
    }

    if (gBrowser.localName == "tabbrowser") {
    var newTab;
        if (theTabPref) {
            // open link in new tab according to TBP
            // FIXED: don't send the referrer, this may be a security hole and
            // clutters server logs; thanks go to jruderman
             //
            newTab = gBrowser.addTab(theURI, null, originCharset, thePostData);
            if ( !BackgroundPref ) {
              gBrowser.selectedTab = newTab;
            }
            if (theEvent.target) {
              markLinkVisited(theURI, theEvent.target);
            }
        }
        else if (theEvent && ('altKey' in theEvent & theEvent.altKey)) {
            // open link in new tab according to keypress
            newTab = gBrowser.addTab(theURI, null, originCharset, thePostData);
            gBrowser.selectedTab = newTab;
            if (theEvent.target) {
              markLinkVisited(theURI, theEvent.target);
            }
        }
        else {
            loadURI(theURI, null, thePostData);
            _content.focus();
            return;
        }

        _content.focus();

    if (theEvent) {
          theEvent.preventDefault();
          theEvent.preventBubble();
          theEvent.preventCapture();
          theEvent.stopPropagation();
    }
        return;
    }
    else {
          loadURI(theURI, null, thePostData);
          _content.focus();
          return;
    }
    return;
}