// code borrowed from Click2Tab by NetOne
// modified by Hemiola SUN
// for restoring the last session when starting the program
////////////////////////////////////////////////////////////////////

const gRDFDataSource = "rdf:local-store";
const gRDFRoot = "http://hemiolapei.free.fr/rdf/all";
const gRDFNodeClosedWindows = gRDFRoot + "/window";
const gRDFLabelClosedTabs = gRDFNodeClosedWindows + "#closedtabs";
const gRDFNodeClosedTabs = gRDFNodeClosedWindows + "/closedtabs";
const gRDFNodeSession = gRDFNodeClosedTabs + "/session";
const gRDFLabelSessionIndex = gRDFNodeSession + "#index";
const gRDFLabelSessionHistory = gRDFNodeSession + "#history";
const gRDFLabelHistoryEntry = gRDFNodeSession + "/history/entry";
const gRDFLabelHistoryEntryTitle = gRDFLabelHistoryEntry + "#title";
const gRDFLabelHistoryEntryURL = gRDFLabelHistoryEntry + "#url";

function deleteNode(rdfNode, dataSource)
{
    if (!dataSource)
    {
        var rdfService = Components.classes["@mozilla.org/rdf/rdf-service;1"]
                                   .getService(Components.interfaces.nsIRDFService);
        dataSource = rdfService.GetDataSource(gRDFDataSource);
    }

    var arcOut = dataSource.ArcLabelsOut(rdfNode);
    while (arcOut.hasMoreElements())
    {
        var aLabel = arcOut.getNext();
        if (aLabel instanceof Components.interfaces.nsIRDFResource)
        {
            var aTarget = dataSource.GetTarget(rdfNode, aLabel, true);
            dataSource.Unassert(rdfNode, aLabel, aTarget);
        }
    }
}

function deleteSubtree(labelRoot, dataSource)
{
    if (!dataSource)
    {
        var rdfService = Components.classes["@mozilla.org/rdf/rdf-service;1"]
                                   .getService(Components.interfaces.nsIRDFService);
        dataSource = rdfService.GetDataSource(gRDFDataSource);
    }

    var allElements = dataSource.GetAllResources();

    while (allElements.hasMoreElements())
    {
        var aResource = allElements.getNext();
        if ((aResource instanceof Components.interfaces.nsIRDFResource)
            && (aResource.Value.indexOf(labelRoot) == 0))
            deleteNode(aResource, dataSource);
    }
}

function numberOfWindows ()
{
  var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService();
  var windowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator);
  var enumerator = windowManagerInterface.getEnumerator("navigator:browser");

  var count = 0;
  while ( enumerator.hasMoreElements() ) {
    count++;
    enumerator.getNext();
  }
  return count;
}

var SessionRestore =
{
    closedTabs : [],
    tabIndex : 0,

    // cache the tab, return value indicate whether the tab is cached
    cacheClosedTabs : function () //browser)
    {
        for (var i = gBrowser.tabContainer.childNodes.length - 1; i >= 0; i--)
          if ( gBrowser.getBrowserAtIndex(i).sessionHistory.count > 0 )
            this.closedTabs.unshift([gBrowser.getBrowserAtIndex(i).sessionHistory]);
        return true;
    },

    clearAllClosedTabs : function ()
    {
        this.closedTabs.splice(0, this.closedTabs.length);
    },

    saveClosedTabs : function ()
    {
        if ( numberOfWindows() )
          return;
        var rdfService = Components.classes["@mozilla.org/rdf/rdf-service;1"]
                                   .getService(Components.interfaces.nsIRDFService);
        var dataSource = rdfService.GetDataSource(gRDFDataSource);
        var rdfNodeClosedWindows = rdfService.GetResource(gRDFNodeClosedWindows);
        var containerUtils = Components.classes["@mozilla.org/rdf/container-utils;1"]
                                       .getService(Components.interfaces.nsIRDFContainerUtils);

        var windowContainer = Components.classes["@mozilla.org/rdf/container;1"]
                                        .createInstance(Components.interfaces.nsIRDFContainer);
        try
        {
            windowContainer.Init(dataSource, rdfNodeClosedWindows);
        }
        catch (ex)
        {
            windowContainer = containerUtils.MakeSeq(dataSource, rdfNodeClosedWindows);
        }

        var windowID = windowContainer.GetCount();
        var rdfNodeThisWindow = rdfService.GetResource(gRDFNodeClosedWindows + windowID);
        // Add the node to the Sequence as early as possible.
        // Note that the first element is 1 and not 0
        windowContainer.InsertElementAt(rdfNodeThisWindow, 1, true);

        var rdfLabelClosedTabs = gRDFNodeClosedWindows + windowID + "/closedtabs";
        var rdfNodeClosedTabs = rdfService.GetResource(rdfLabelClosedTabs);
        var tabContainer = Components.classes["@mozilla.org/rdf/container;1"]
                                     .createInstance(Components.interfaces.nsIRDFContainer);
        try
        {
            tabContainer.Init(dataSource, rdfNodeClosedTabs);
        }
        catch (ex)
        {
            tabContainer = containerUtils.MakeSeq(dataSource, rdfNodeClosedTabs);
        }

        for (var i = 0; i < this.closedTabs.length; i++)
        {
            var sessionHistory = this.closedTabs[i][0];

            var rdfLabelSession = rdfLabelClosedTabs + "/session" + i;
            var rdfNodeSession = rdfService.GetResource(rdfLabelSession);

            var rdfLabelSessionHistory = rdfLabelSession + "/history";
            var rdfNodeSessionHistory = rdfService.GetResource(rdfLabelSessionHistory);
            var entryContainer = Components.classes["@mozilla.org/rdf/container;1"]
                                           .createInstance(Components.interfaces.nsIRDFContainer);
            try
            {
                entryContainer.Init(dataSource, rdfNodeSessionHistory);
            }
            catch (ex)
            {
                entryContainer = containerUtils.MakeSeq(dataSource, rdfNodeSessionHistory);
            }

            for (var j = 0; j < sessionHistory.count; j++)
            {
                var rdfNodeHistoryEntry = rdfService.GetResource(rdfLabelSessionHistory + "/entry" + j);
                var historyEntry = sessionHistory.getEntryAtIndex(j, false);
                dataSource.Assert(rdfNodeHistoryEntry,
                                  rdfService.GetResource(gRDFLabelHistoryEntryTitle),
                                  rdfService.GetLiteral(historyEntry.title),
                                  true);
                dataSource.Assert(rdfNodeHistoryEntry,
                                  rdfService.GetResource(gRDFLabelHistoryEntryURL),
                                  rdfService.GetLiteral(historyEntry.URI.spec),
                                  true);
                entryContainer.AppendElement(rdfNodeHistoryEntry);
            } // end of "for (var j = 0; j < sessionHistory.count; j++)"

            dataSource.Assert(rdfNodeSession,
                              rdfService.GetResource(gRDFLabelSessionHistory),
                              rdfNodeSessionHistory,
                              true);
            dataSource.Assert(rdfNodeSession,
                              rdfService.GetResource(gRDFLabelSessionIndex),
                              rdfService.GetIntLiteral(sessionHistory.index),
                              true);

            tabContainer.AppendElement(rdfNodeSession);
        } // end of "for (var i = 0; i < this.closedTabs.length; i++)"
        dataSource.Assert(rdfNodeThisWindow,
                          rdfService.GetResource(gRDFLabelClosedTabs),
                          rdfNodeClosedTabs,
                          true);

        // Since it's possible that there are multiple windows open at the same time,
        // it would be better to save the RDF file immediately
        dataSource.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();
    }, // end of "saveClosedTabs : function ()"

    loadClosedTabs : function ()
    {
        if ( numberOfWindows() > 1 )
          return;
        var overwrite = window.arguments[0] == gHomeButton.getHomePage();
        // bug: if the page demanded equal the homepage, Session Restore will still overwrite it...

        var rdfService = Components.classes["@mozilla.org/rdf/rdf-service;1"]
                                   .getService(Components.interfaces.nsIRDFService);
        var dataSource = rdfService.GetDataSource(gRDFDataSource);
        var rdfNodeClosedWindows = rdfService.GetResource(gRDFNodeClosedWindows);
        var containerUtils = Components.classes["@mozilla.org/rdf/container-utils;1"]
                                       .getService(Components.interfaces.nsIRDFContainerUtils);

        var windowContainer = Components.classes["@mozilla.org/rdf/container;1"]
                                        .createInstance(Components.interfaces.nsIRDFContainer);
        try
        {
            windowContainer.Init(dataSource, rdfNodeClosedWindows);
        }
        catch (ex)
        {
            windowContainer = containerUtils.MakeSeq(dataSource, rdfNodeClosedWindows);
        }

        var windowEnum = windowContainer.GetElements();
        if (windowEnum.hasMoreElements())
        {
            var rdfNodeLastWindow = windowEnum.getNext();
            if (rdfNodeLastWindow instanceof Components.interfaces.nsIRDFResource)
            {
                var rdfLabelClosedTabs = rdfService.GetResource(gRDFLabelClosedTabs);
                var rdfNodeClosedTabs = dataSource.GetTarget(rdfNodeLastWindow, rdfLabelClosedTabs, true);
                if (rdfNodeClosedTabs instanceof Components.interfaces.nsIRDFResource)
                {
                    var tabContainer = Components.classes["@mozilla.org/rdf/container;1"]
                                                 .createInstance(Components.interfaces.nsIRDFContainer);
                    try
                    {
                        tabContainer.Init(dataSource, rdfNodeClosedTabs);
                    }
                    catch (ex)
                    {
                        tabContainer = containerUtils.MakeSeq(dataSource, rdfNodeClosedTabs);
                    }

                    var sessionEnum = tabContainer.GetElements();
                    while (sessionEnum.hasMoreElements())
                    {
                        var rdfNodeSession = sessionEnum.getNext();
                        if (rdfNodeSession instanceof Components.interfaces.nsIRDFResource)
                            this.loadOneTab(rdfService, dataSource, containerUtils, rdfNodeSession, overwrite);
                    }
                } // end of "if (rdfNodeClosedTabs instanceof Components.interfaces.nsIRDFResource)"
                deleteSubtree(rdfNodeLastWindow.Value, dataSource);
                windowContainer.RemoveElement(rdfNodeLastWindow, true);
                // delete the sequence if blank
                if (!windowContainer.GetCount())
                    deleteNode(rdfNodeClosedWindows, dataSource);

                // it would be better to save the RDF file immediately
                dataSource.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();

            } // end of "if (rdfNodeLastWindow instanceof Components.interfaces.nsIRDFResource)"
        } // end of "if (windowEnum.hasMoreElements())"
if (gBrowser.mTabContainer.childNodes.length > 1)
if (gBrowser.mTabContainer.childNodes[0].label == gBrowser.mStringBundle.getString("tabs.untitled"))
gBrowser.removeTab(gBrowser.mTabContainer.childNodes[0]);
var aIndexPref = "extensions.tabmix.sessionsRestore.index";
var aIndex = gPref.prefHasUserValue(aIndexPref) ? gPref.getIntPref(aIndexPref): 0 ;
gBrowser.mTabContainer.selectedIndex = aIndex ;
if (aIndex!=0) gBrowser.mTabContainer.childNodes[0].removeAttribute("selected");
window.setTimeout( function () {checkBeforeAndAfter(); }, 0 );
    }, // end of loadClosedTabs : function ()"

    loadOneTab : function(rdfService, dataSource, containerUtils, rdfNodeSession, overwrite)
    {
        var rdfLabelSessionHistory = rdfService.GetResource(gRDFLabelSessionHistory);
        var rdfNodeSessionHistory = dataSource.GetTarget(rdfNodeSession, rdfLabelSessionHistory, true);
        if (rdfNodeSessionHistory instanceof Components.interfaces.nsIRDFResource)
        {
            var entryContainer = Components.classes["@mozilla.org/rdf/container;1"]
                                           .createInstance(Components.interfaces.nsIRDFContainer);
            try
            {
                entryContainer.Init(dataSource, rdfNodeSessionHistory);
            }
            catch (ex)
            {
                entryContainer = containerUtils.MakeSeq(dataSource, rdfNodeSessionHistory);
            }

            var ioService = Components.classes["@mozilla.org/network/io-service;1"]
                                      .getService(Components.interfaces.nsIIOService);

            var sessionHistory = Components.classes["@mozilla.org/browser/shistory;1"]
                                           .createInstance(Components.interfaces.nsISHistory);
            var sHistoryInternal = sessionHistory;
            sHistoryInternal = sHistoryInternal.QueryInterface(Components.interfaces.nsISHistoryInternal);

            var entryEnum = entryContainer.GetElements();
            while (entryEnum.hasMoreElements())
            {
                var rdfNodeHistoryEntry = entryEnum.getNext();
                if (rdfNodeHistoryEntry instanceof Components.interfaces.nsIRDFResource)
                {
                    var historyEntry = Components.classes["@mozilla.org/browser/session-history-entry;1"]
                                                 .createInstance(Components.interfaces.nsISHEntry);

                    var rdfLabelHistoryEntryTitle = rdfService.GetResource(gRDFLabelHistoryEntryTitle);
                    var rdfNodeHistoryEntryTitle = dataSource.GetTarget(rdfNodeHistoryEntry, rdfLabelHistoryEntryTitle, true);
                    var entryTitle = "";
                    if (rdfNodeHistoryEntryTitle instanceof Components.interfaces.nsIRDFLiteral)
                        entryTitle = rdfNodeHistoryEntryTitle.Value;
                    (typeof(historyEntry.SetTitle)!='undefined') ? historyEntry.SetTitle(entryTitle) : historyEntry.setTitle(entryTitle);

                    var rdfLabelHistoryEntryURL = rdfService.GetResource(gRDFLabelHistoryEntryURL);
                    var rdfNodeHistoryEntryURL = dataSource.GetTarget(rdfNodeHistoryEntry, rdfLabelHistoryEntryURL, true);
                    var entryURL = "about:blank";
                    if (rdfNodeHistoryEntryURL instanceof Components.interfaces.nsIRDFLiteral)
                        entryURL = rdfNodeHistoryEntryURL.Value;
                    (typeof(historyEntry.SetURI)!='undefined') ? historyEntry.SetURI(ioService.newURI(entryURL, null, null)) : historyEntry.setURI(ioService.newURI(entryURL, null, null));

                    sHistoryInternal.addEntry(historyEntry, true);
                } // end of "if (rdfNodeHistoryEntry instanceof Components.interfaces.nsIRDFResource)"
            } // end of "while (entryEnum.hasMoreElements())"

            var rdfLabelSessionIndex = rdfService.GetResource(gRDFLabelSessionIndex);
            var rdfNodeSessionIndex = dataSource.GetTarget(rdfNodeSession, rdfLabelSessionIndex, true);
            var sessionIndex = sessionHistory.index;
            if (rdfNodeSessionIndex instanceof Components.interfaces.nsIRDFInt)
                sessionIndex = rdfNodeSessionIndex.Value;

            // load the tabs and their history, code added by Hemiola SUN
            var aTab;
            if ( !overwrite || !gBrowser.mTabContainer.childNodes[this.tabIndex] ) {
              aTab = gBrowser.addTab("about:blank");
              this.tabIndex = aTab.ordinal;
            }
            else {
              aTab = gBrowser.mTabContainer.childNodes[this.tabIndex];
              gBrowser.getBrowserForTab(aTab).webNavigation.sessionHistory =
                Components.classes["@mozilla.org/browser/shistory;1"]
                .createInstance(Components.interfaces.nsISHistory);
            }

            var newHistory = gBrowser.getBrowserForTab(aTab).webNavigation.sessionHistory;

            newHistory.QueryInterface(Components.interfaces.nsISHistoryInternal);
            newHistory.count = 0;
            for (var i = 0; i < sessionHistory.count; i++)
            {
                newHistory.addEntry(sessionHistory.getEntryAtIndex(i, false), true);
            }
            var ind = ( sessionIndex > -1 && sessionIndex < sessionHistory.count ) ?
                        sessionIndex : sessionHistory.count - 1;
            gBrowser.getBrowserForTab(aTab).webNavigation.gotoIndex(ind);
            this.tabIndex++;

  } // end of "if (rdfNodeSessionHistory instanceof Components.interfaces.nsIRDFResource)"
     window.setTimeout( function () {tabBarScrollStatus(); }, 0 ); 
    } // end of "loadOneTab : function(rdfService, rdfDatasource, rdfNodeSession)"
};